@@ -42,6 +42,7 @@
 #include <kstandarddirs.h>
 #include <kautostart.h>
 #include <KDebug>
+#include <KNotification>
 #include <KLibrary>
 #include <KColorScheme>
 #include <KStandardDirs>
@@ -259,9 +260,14 @@ KCMStyle::KCMStyle( QWidget* parent, const QVariantList& )
     connect(fineTuningUi.comboGraphicEffectsLevel, SIGNAL(activated(int)),   this, SLOT(setEffectsDirty()));
     connect(fineTuningUi.comboToolbarIcons,    SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
     connect(fineTuningUi.comboSecondaryToolbarIcons,    SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
+    connect(fineTuningUi.comboMenubarStyle,    SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
 
     addWhatsThis();
 
+    if (!QFile::exists(QLibraryInfo::location(QLibraryInfo::PluginsPath) + "/menubar/libappmenu-qt.so")) {
+        fineTuningUi.menubarBox->hide();
+    }
+
     // Insert the pages into the tabWidget
     tabWidget->addTab(page1, i18nc("@title:tab", "&Applications"));
     tabWidget->addTab(page2, i18nc("@title:tab", "&Fine Tuning"));
@@ -400,8 +406,71 @@ void KCMStyle::save()
                             toolbarButtonText(fineTuningUi.comboToolbarIcons->currentIndex()));
     toolbarStyleGroup.writeEntry("ToolButtonStyleOtherToolbars",
                             toolbarButtonText(fineTuningUi.comboSecondaryToolbarIcons->currentIndex()));
+
+    // menubar page
+    KConfigGroup menuBarStyleGroup(&_config, "Appmenu Style");
+
+    // load kded module if needed
+    bool load = false;
+    QList<QVariant> args;
+
+    QString style = menuBarStyleText(fineTuningUi.comboMenubarStyle->currentIndex());
+
+    QString previous = menuBarStyleGroup.readEntry("Style", "InApplication");
+    menuBarStyleGroup.writeEntry("Style", style);
     _config.sync();
 
+    QDBusMessage method = QDBusMessage::createMethodCall("org.kde.kded",
+                                                         "/modules/appmenu",
+                                                         "org.kde.kded",
+                                                         "reconfigure");
+    QDBusConnection::sessionBus().asyncCall(method);
+
+    if (previous == "InApplication" && style != "InApplication") {
+        load = true;
+        KNotification *notification = new KNotification("reload", 0);
+        notification->setComponentData(KComponentData("kcmstyle"));
+        notification->setText(i18n("Settings changes will take effect only on application restart"));
+        notification->sendEvent();
+    }
+
+    // If user select ButtonVertical, we add (if needed) a button to titlebar
+    if (style == "ButtonVertical") {
+        KConfig _kwinConfig("kwinrc", KConfig::NoGlobals);
+        KConfigGroup kwinConfig(&_kwinConfig, "Style");
+        QString buttonsOnLeft = kwinConfig.readEntry("ButtonsOnLeft", "");
+        QString buttonsOnRight = kwinConfig.readEntry("ButtonsOnRight", "IMX");
+        qDebug() << buttonsOnLeft << buttonsOnRight;
+        if (!buttonsOnLeft.contains("N") && !buttonsOnRight.contains("N")) {
+            buttonsOnLeft = "N" + buttonsOnLeft;
+        }
+        kwinConfig.writeEntry("ButtonsOnLeft", buttonsOnLeft);
+        kwinConfig.writeEntry("CustomButtonPositions", "true");
+    }
+
+    args = QList<QVariant>() << "appmenu" << load;
+    method = QDBusMessage::createMethodCall("org.kde.kded",
+                                            "/kded",
+                                            "org.kde.kded",
+                                            "setModuleAutoloading");
+    method.setArguments(args);
+    QDBusConnection::sessionBus().asyncCall(method);
+
+    args = QList<QVariant>() << "appmenu";
+    if (load) {
+        method = QDBusMessage::createMethodCall("org.kde.kded",
+                                                "/kded",
+                                                "org.kde.kded",
+                                                "loadModule");
+    } else {
+        method = QDBusMessage::createMethodCall("org.kde.kded",
+                                                "/kded",
+                                                "org.kde.kded",
+                                                "unloadModule");
+    }
+    method.setArguments(args);
+    QDBusConnection::sessionBus().asyncCall(method);
+
     // Export the changes we made to qtrc, and update all qt-only
     // applications on the fly, ensuring that we still follow the user's
     // export fonts/colors settings.
@@ -487,6 +556,7 @@ void KCMStyle::defaults()
     // Effects
     fineTuningUi.comboToolbarIcons->setCurrentIndex(toolbarButtonIndex("TextBesideIcon"));
     fineTuningUi.comboSecondaryToolbarIcons->setCurrentIndex(toolbarButtonIndex("TextBesideIcon"));
+    fineTuningUi.comboMenubarStyle->setCurrentIndex(menuBarStyleIndex("InApplication"));
     fineTuningUi.cbIconsOnButtons->setChecked(true);
     fineTuningUi.cbIconsInMenus->setChecked(true);
     fineTuningUi.comboGraphicEffectsLevel->setCurrentIndex(fineTuningUi.comboGraphicEffectsLevel->findData(((int) KGlobalSettings::graphicEffectsLevelDefault())));
@@ -708,6 +778,33 @@ int KCMStyle::toolbarButtonIndex(const QString &text)
     return 0;
 }
 
+QString KCMStyle::menuBarStyleText(int index)
+{
+    switch (index) {
+        case 1:
+            return "ButtonVertical";
+        case 2:
+            return "TopMenuBar";
+        case 3:
+            return "Others";
+    }
+
+    return "InApplication";
+}
+
+int KCMStyle::menuBarStyleIndex(const QString &text)
+{
+    if (text == "ButtonVertical") {
+        return 1;
+    } else if (text == "TopMenuBar") {
+        return 2;
+    } else if (text == "Others") {
+        return 3;
+    }
+
+    return 0;
+}
+
 void KCMStyle::loadEffects( KConfig& config )
 {
     // KDE's Part via KConfig
@@ -718,6 +815,10 @@ void KCMStyle::loadEffects( KConfig& config )
     tbIcon = configGroup.readEntry("ToolButtonStyleOtherToolbars", "TextBesideIcon");
     fineTuningUi.comboSecondaryToolbarIcons->setCurrentIndex(toolbarButtonIndex(tbIcon));
 
+    configGroup = config.group("Appmenu Style");
+    QString menuBarStyle = configGroup.readEntry("Style", "InApplication");
+    fineTuningUi.comboMenubarStyle->setCurrentIndex(menuBarStyleIndex(menuBarStyle));
+
     configGroup = config.group("KDE");
     fineTuningUi.cbIconsOnButtons->setChecked(configGroup.readEntry("ShowIconsOnPushButtons", true));
     fineTuningUi.cbIconsInMenus->setChecked(configGroup.readEntry("ShowIconsInMenuItems", true));
