@@ -39,6 +39,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "atoms.h"
 #include "group.h"
 #include "rules.h"
+#include "useractions.h"
 #include <QX11Info>
 
 namespace KWin
@@ -228,6 +229,9 @@ void Workspace::setActiveClient(Client* c, allowed_t)
 
     if (active_popup && active_popup_client != c && set_active_client_recursion == 0)
         closeActivePopup();
+    if (m_userActionsMenu->hasClient() && !m_userActionsMenu->isMenuClient(c) && set_active_client_recursion == 0) {
+        m_userActionsMenu->close();
+    }
     StackingUpdatesBlocker blocker(this);
     ++set_active_client_recursion;
     updateFocusMousePosition(cursorPos());
@@ -414,7 +418,7 @@ Client *Workspace::clientUnderMouse(int screen) const
         }
 
         // rule out clients which are not really visible.
-        // the screen test is rather superflous for xrandr & twinview since the geometry would differ -> TODO: might be dropped
+        // the screen test is rather superfluous for xrandr & twinview since the geometry would differ -> TODO: might be dropped
         if (!(client->isShown(false) && client->isOnCurrentDesktop() &&
                 client->isOnCurrentActivity() && client->isOnScreen(screen)))
             continue;
@@ -453,16 +457,24 @@ bool Workspace::activateNextClient(Client* c)
 
     Client* get_focus = NULL;
 
-    if (options->isNextFocusPrefersMouse()) {
+    // precedence on keeping the current tabgroup active. to the user that's the same window
+    if (c && c->tabGroup()) {
+        if (c == c->tabGroup()->current())
+            c->tabGroup()->activateNext();
+        get_focus = c->tabGroup()->current();
+        if (get_focus == c) // single tab case - should not happen
+            get_focus = NULL;
+    }
+
+    if (!get_focus && options->isNextFocusPrefersMouse()) {
         get_focus = clientUnderMouse(c ? c->screen() : activeScreen());
         if (get_focus && (get_focus == c || get_focus->isDesktop())) {
             // should rather not happen, but it cannot get the focus. rest of usability is tested above
-            get_focus = 0;
+            get_focus = NULL;
         }
     }
 
     if (!get_focus) { // no suitable window under the mouse -> find sth. else
-
         // first try to pass the focus to the (former) active clients leader
         if (c  && (get_focus = c->transientFor()) && isUsableFocusCandidate(get_focus, c, options->isSeparateScreenFocus())) {
             raiseClient(get_focus);   // also raise - we don't know where it came from
@@ -505,10 +517,10 @@ void Workspace::setCurrentScreen(int new_screen)
         Client* ci = focus_chain[ currentDesktop()].at(i);
         if (!ci->isShown(false) || !ci->isOnCurrentDesktop() || !ci->isOnCurrentActivity())
             continue;
-        if (ci->screen() == new_screen) {
-            get_focus = ci;
-            break;
-        }
+        if (!ci->screen() == new_screen)
+            continue;
+        get_focus = ci;
+        break;
     }
     if (get_focus == NULL)
         get_focus = findDesktop(true, currentDesktop());
@@ -869,7 +881,6 @@ void Client::setActive(bool act)
     emit activeChanged();
     updateMouseGrab();
     updateUrgency(); // demand attention again if it's still urgent
-    workspace()->checkUnredirect();
 }
 
 void Client::startupIdChanged()
