@@ -26,6 +26,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDateTime>
 #include <QProcess>
 #include <QPaintEngine>
+
+#ifdef KWIN_BUILD_SCRIPTING
+#include <QScriptEngine>
+#include <QScriptProgram>
+#endif
+
 #include <unistd.h>
 #include <kstandarddirs.h>
 #include <QWhatsThis>
@@ -35,6 +41,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <signal.h>
 
 #include "bridge.h"
+#include "composite.h"
 #include "group.h"
 #include "workspace.h"
 #include "atoms.h"
@@ -92,6 +99,7 @@ Client::Client(Workspace* ws)
     , bridge(new Bridge(this))
     , move_resize_grab_window(None)
     , move_resize_has_keyboard_grab(false)
+    , m_managed(false)
     , transient_for (NULL)
     , transient_for_id(None)
     , original_transient_for_id(None)
@@ -122,12 +130,15 @@ Client::Client(Workspace* ws)
     , padding_bottom(0)
     , sm_stacking_order(-1)
     , demandAttentionKNotifyTimer(NULL)
-    , m_responsibleForDecoPixmap(false)
     , paintRedirector(0)
     , m_firstInTabBox(false)
     , electricMaximizing(false)
     , activitiesDefined(false)
     , needsSessionInteract(false)
+    , needsXWindowMove(false)
+#ifdef KWIN_BUILD_KAPPMENU
+    , m_menuAvailable(false)
+#endif
     , input_window(None)
 {
     // TODO: Do all as initialization
@@ -346,7 +357,7 @@ void Client::updateInputWindow()
 {
     QRegion region;
 
-    if (!noBorder() && dynamic_cast<KDecorationUnstable*>(decoration)) {
+    if (!noBorder()) {
         // This function is implemented as a slot to avoid breaking binary
         // compatibility
         QMetaObject::invokeMethod(decoration, "region", Qt::DirectConnection,
@@ -380,6 +391,8 @@ void Client::updateInputWindow()
         input_window = XCreateWindow(display(), rootWindow(), bounds.x(), bounds.y(),
                                      bounds.width(), bounds.height(), 0, 0,
                                      InputOnly, 0, CWEventMask | CWOverrideRedirect, &attr);
+        if (mapping_state == Mapped)
+            XMapWindow(display(), inputId());
     } else {
         XMoveResizeWindow(display(), input_window, bounds.x(), bounds.y(),
                           bounds.width(), bounds.height());
@@ -400,6 +413,12 @@ void Client::updateDecoration(bool check_workspace_pos, bool force)
     if (!noBorder()) {
         setMask(QRegion());  // Reset shape mask
         decoration = workspace()->createDecoration(bridge);
+#ifdef KWIN_BUILD_KAPPMENU
+        connect(this, SIGNAL(showRequest()), decoration, SIGNAL(showRequest()));
+        connect(this, SIGNAL(appMenuAvailable()), decoration, SIGNAL(appMenuAvailable()));
+        connect(this, SIGNAL(appMenuUnavailable()), decoration, SIGNAL(appMenuUnavailable()));
+        connect(this, SIGNAL(menuHidden()), decoration, SIGNAL(menuHidden()));
+#endif
         // TODO: Check decoration's minimum size?
         decoration->init();
         decoration->widget()->installEventFilter(this);
@@ -412,11 +431,10 @@ void Client::updateDecoration(bool check_workspace_pos, bool force)
         XMoveWindow(display(), decoration->widget()->winId(), -padding_left, -padding_top);
         move(calculateGravitation(false));
         plainResize(sizeForClientSize(clientSize()), ForceGeometrySet);
-        paintRedirector = new PaintRedirector(decoration->widget());
-        connect(paintRedirector, SIGNAL(paintPending()), SLOT(repaintDecorationPending()));
-        resizeDecorationPixmaps();
-        if (compositing())
+        if (Compositor::compositing()) {
+            paintRedirector = new PaintRedirector(this, decoration->widget());
             discardWindowPixmap();
+        }
         emit geometryShapeChanged(this, oldgeom);
     } else
         destroyDecoration();
@@ -435,21 +453,12 @@ void Client::destroyDecoration()
     if (decoration != NULL) {
         delete decoration;
         decoration = NULL;
+        paintRedirector = NULL;
         QPoint grav = calculateGravitation(true);
         border_left = border_right = border_top = border_bottom = 0;
         setMask(QRegion());  // Reset shape mask
         plainResize(sizeForClientSize(clientSize()), ForceGeometrySet);
         move(grav);
-        delete paintRedirector;
-        paintRedirector = NULL;
-        if (m_responsibleForDecoPixmap) {
-            XFreePixmap(display(), decorationPixmapTop.handle());
-            XFreePixmap(display(), decorationPixmapBottom.handle());
-            XFreePixmap(display(), decorationPixmapLeft.handle());
-            XFreePixmap(display(), decorationPixmapRight.handle());
-            m_responsibleForDecoPixmap = false;
-        }
-        decorationPixmapLeft = decorationPixmapRight = decorationPixmapTop = decorationPixmapBottom = QPixmap();
         if (compositing())
             discardWindowPixmap();
         if (!deleting) {
@@ -541,162 +550,6 @@ QRegion Client::decorationPendingRegion() const
     return paintRedirector->scheduledRepaintRegion().translated(x() - padding_left, y() - padding_top);
 }
 
-void Client::repaintDecorationPending()
-{
-    if (compositing()) {
-        // The scene will update the decoration pixmaps in the next painting pass
-        // if it has not been already repainted before
-        const QRegion r = paintRedirector->scheduledRepaintRegion();
-        if (!r.isEmpty()) {
-            addRepaint(r.translated(-padding_left,-padding_top));
-        }
-    }
-}
-
-bool Client::decorationPixmapRequiresRepaint()
-{
-    if (!paintRedirector)
-        return false;
-    QRegion r = paintRedirector->pendingRegion();
-    return !r.isEmpty();
-}
-
-void Client::ensureDecorationPixmapsPainted()
-{
-    if (!paintRedirector || !compositing())
-        return;
-
-    QRegion r = paintRedirector->pendingRegion();
-    if (r.isEmpty())
-        return;
-
-    QPixmap p = paintRedirector->performPendingPaint();
-
-    QRect lr, rr, tr, br;
-    layoutDecorationRects(lr, tr, rr, br, DecorationRelative);
-
-    repaintDecorationPixmap(decorationPixmapLeft, lr, p, r);
-    repaintDecorationPixmap(decorationPixmapRight, rr, p, r);
-    repaintDecorationPixmap(decorationPixmapTop, tr, p, r);
-    repaintDecorationPixmap(decorationPixmapBottom, br, p, r);
-
-    XSync(display(), false);
-}
-
-void Client::repaintDecorationPixmap(QPixmap& pix, const QRect& r, const QPixmap& src, QRegion reg)
-{
-    if (!r.isValid())
-        return;
-    QRect b = reg.boundingRect();
-    reg &= r;
-    if (reg.isEmpty())
-        return;
-    QPainter pt(&pix);
-    pt.translate(-r.topLeft());
-    pt.setCompositionMode(QPainter::CompositionMode_Source);
-    pt.setClipRegion(reg);
-    pt.drawPixmap(b.topLeft(), src);
-    pt.end();
-}
-
-void Client::resizeDecorationPixmaps()
-{
-    if (!compositing()) {
-        // compositing disabled - we render directly on screen
-        triggerDecorationRepaint();
-        return;
-    }
-    QRect lr, rr, tr, br;
-    layoutDecorationRects(lr, tr, rr, br, DecorationRelative);
-
-    if (decorationPixmapTop.size() != tr.size()) {
-        if (m_responsibleForDecoPixmap && !decorationPixmapTop.isNull() &&
-              decorationPixmapTop.paintEngine()->type() == QPaintEngine::X11) {
-            XFreePixmap(display(), decorationPixmapTop.handle());
-        }
-
-        if (workspace()->compositingActive() && effects->compositingType() == OpenGLCompositing) {
-            decorationPixmapTop = QPixmap(tr.size());
-            m_responsibleForDecoPixmap = false;
-        } else {
-            Pixmap xpix = XCreatePixmap(QX11Info::display(), rootWindow(),
-                                        tr.size().width(), tr.height(),
-                                        32);
-            decorationPixmapTop = QPixmap::fromX11Pixmap(xpix, QPixmap::ExplicitlyShared);
-            decorationPixmapTop.fill(Qt::transparent);
-            m_responsibleForDecoPixmap= true;
-        }
-    }
-
-    if (decorationPixmapBottom.size() != br.size()) {
-        if (m_responsibleForDecoPixmap && !decorationPixmapBottom.isNull() &&
-              decorationPixmapBottom.paintEngine()->type() == QPaintEngine::X11) {
-            XFreePixmap(display(), decorationPixmapBottom.handle());
-        }
-
-        if (workspace()->compositingActive() && effects->compositingType() == OpenGLCompositing) {
-            decorationPixmapBottom = QPixmap(br.size());
-            m_responsibleForDecoPixmap = false;
-        } else {
-            Pixmap xpix = XCreatePixmap(QX11Info::display(), rootWindow(),
-                                        br.size().width(), br.height(),
-                                        32);
-            decorationPixmapBottom = QPixmap::fromX11Pixmap(xpix, QPixmap::ExplicitlyShared);
-            decorationPixmapBottom.fill(Qt::transparent);
-            m_responsibleForDecoPixmap = true;
-        }
-    }
-
-    if (decorationPixmapLeft.size() != lr.size()) {
-        if (m_responsibleForDecoPixmap && !decorationPixmapLeft.isNull() &&
-              decorationPixmapLeft.paintEngine()->type() == QPaintEngine::X11) {
-            XFreePixmap(display(), decorationPixmapLeft.handle());
-        }
-
-        if (workspace()->compositingActive() && effects->compositingType() == OpenGLCompositing) {
-            decorationPixmapLeft = QPixmap(lr.size());
-            m_responsibleForDecoPixmap = false;
-        } else {
-            Pixmap xpix = XCreatePixmap(QX11Info::display(), rootWindow(),
-                                        lr.size().width(), lr.height(),
-                                        32);
-            decorationPixmapLeft = QPixmap::fromX11Pixmap(xpix, QPixmap::ExplicitlyShared);
-            decorationPixmapLeft.fill(Qt::transparent);
-            m_responsibleForDecoPixmap = true;
-        }
-    }
-
-    if (decorationPixmapRight.size() != rr.size()) {
-        if (m_responsibleForDecoPixmap && !decorationPixmapRight.isNull() &&
-              decorationPixmapRight.paintEngine()->type() == QPaintEngine::X11) {
-            XFreePixmap(display(), decorationPixmapRight.handle());
-        }
-
-        if (workspace()->compositingActive() && effects->compositingType() == OpenGLCompositing) {
-            decorationPixmapRight = QPixmap(rr.size());
-            m_responsibleForDecoPixmap = false;
-        } else {
-            Pixmap xpix = XCreatePixmap(QX11Info::display(), rootWindow(),
-                                        rr.size().width(), rr.height(),
-                                        32);
-            decorationPixmapRight = QPixmap::fromX11Pixmap(xpix, QPixmap::ExplicitlyShared);
-            decorationPixmapRight.fill(Qt::transparent);
-            m_responsibleForDecoPixmap = true;
-        }
-    }
-
-#ifdef HAVE_XRENDER
-    if (Extensions::renderAvailable()) {
-        // Make sure the pixmaps are created with alpha channels
-        decorationPixmapLeft.fill(Qt::transparent);
-        decorationPixmapRight.fill(Qt::transparent);
-        decorationPixmapTop.fill(Qt::transparent);
-        decorationPixmapBottom.fill(Qt::transparent);
-    }
-#endif
-    triggerDecorationRepaint();
-}
-
 QRect Client::transparentRect() const
 {
     if (isShade())
@@ -779,8 +632,10 @@ void Client::resizeDecoration(const QSize& s)
     if (oldSize == newSize) {
         QResizeEvent e(newSize, oldSize);
         QApplication::sendEvent(decoration->widget(), &e);
-    } else { // oldSize != newSize
-        resizeDecorationPixmaps();
+    } else if (paintRedirector) { // oldSize != newSize
+        paintRedirector->resizePixmaps();
+    } else {
+        triggerDecorationRepaint();
     }
     updateInputWindow();
 }
@@ -907,7 +762,8 @@ void Client::setMask(const QRegion& reg, int mode)
     }
     if (shape()) {
         // The rest of the applyign using a temporary window
-        XRectangle rec = { 0, 0, clientSize().width(), clientSize().height() };
+        XRectangle rec = { 0, 0, static_cast<unsigned short>(clientSize().width()),
+                           static_cast<unsigned short>(clientSize().height()) };
         XShapeCombineRectangles(display(), shape_helper_window, ShapeBounding,
                                 clientPos().x(), clientPos().y(), &rec, 1, ShapeSubtract, Unsorted);
         XShapeCombineShape(display(), shape_helper_window, ShapeBounding,
@@ -1273,7 +1129,9 @@ void Client::internalShow(allowed_t)
             XMapWindow(display(), inputId());
         updateHiddenPreview();
     }
-    workspace()->checkUnredirect();
+    if (Compositor::isCreated()) {
+        Compositor::self()->checkUnredirect();
+    }
 }
 
 void Client::internalHide(allowed_t)
@@ -1288,7 +1146,9 @@ void Client::internalHide(allowed_t)
         updateHiddenPreview();
     addWorkspaceRepaint(visibleRect());
     workspace()->clientHidden(this);
-    workspace()->checkUnredirect();
+    if (Compositor::isCreated()) {
+        Compositor::self()->checkUnredirect();
+    }
 }
 
 void Client::internalKeep(allowed_t)
@@ -1305,7 +1165,9 @@ void Client::internalKeep(allowed_t)
     updateHiddenPreview();
     addWorkspaceRepaint(visibleRect());
     workspace()->clientHidden(this);
-    workspace()->checkUnredirect();
+    if (Compositor::isCreated()) {
+        Compositor::self()->checkUnredirect();
+    }
 }
 
 /**
@@ -1855,41 +1717,72 @@ QChar PDF(0x202C);
 
 void Client::setCaption(const QString& _s, bool force)
 {
-    QString s = _s;
-    if (s != cap_normal || force) {
-        bool reset_name = force;
-        for (int i = 0; i < s.length(); ++i)
-            if (!s[i].isPrint())
-                s[i] = QChar(' ');
-        cap_normal = s;
-        bool was_suffix = (!cap_suffix.isEmpty());
-        QString machine_suffix;
+    if (!force && _s == cap_normal)
+        return;
+    QString s(_s);
+    for (int i = 0; i < s.length(); ++i)
+        if (!s[i].isPrint())
+            s[i] = QChar(' ');
+    cap_normal = s;
+#ifdef KWIN_BUILD_SCRIPTING
+    if (options->condensedTitle()) {
+        static QScriptEngine engine;
+        static QScriptProgram stripTitle;
+        static QScriptValue script;
+        if (stripTitle.isNull()) {
+            const QString scriptFile = KStandardDirs::locate("data", QLatin1String(KWIN_NAME) + "/stripTitle.js");
+            if (!scriptFile.isEmpty()) {
+                QFile f(scriptFile);
+                if (f.open(QIODevice::ReadOnly|QIODevice::Text)) {
+                    f.reset();
+                    stripTitle = QScriptProgram(QString::fromLocal8Bit(f.readAll()), "stripTitle.js");
+                    f.close();
+                }
+            }
+            if (stripTitle.isNull())
+                stripTitle = QScriptProgram("(function(title, wm_name, wm_class){ return title ; })", "stripTitle.js");
+            script = engine.evaluate(stripTitle);
+        }
+        QScriptValueList args;
+        args << _s << QString(resourceName()) << QString(resourceClass());
+        s = script.call(QScriptValue(), args).toString();
+    }
+#endif
+    if (!force && s == cap_deco)
+        return;
+    cap_deco = s;
+
+    bool reset_name = force;
+    bool was_suffix = (!cap_suffix.isEmpty());
+    cap_suffix.clear();
+    QString machine_suffix;
+    if (!options->condensedTitle()) { // machine doesn't qualify for "clean"
         if (wmClientMachine(false) != "localhost" && !isLocalMachine(wmClientMachine(false)))
             machine_suffix = QString(" <@") + wmClientMachine(true) + '>' + LRM;
-        QString shortcut_suffix = !shortcut().isEmpty() ? (" {" + shortcut().toString() + '}') : QString();
-        cap_suffix = machine_suffix + shortcut_suffix;
-        if ((!isSpecialWindow() || isToolbar()) && workspace()->findClient(FetchNameInternalPredicate(this))) {
-            int i = 2;
-            do {
-                cap_suffix = machine_suffix + " <" + QString::number(i) + '>' + LRM + shortcut_suffix;
-                i++;
-            } while (workspace()->findClient(FetchNameInternalPredicate(this)));
-            info->setVisibleName(caption().toUtf8());
-            reset_name = false;
-        }
-        if ((was_suffix && cap_suffix.isEmpty()) || reset_name) {
-            // If it was new window, it may have old value still set, if the window is reused
-            info->setVisibleName("");
-            info->setVisibleIconName("");
-        } else if (!cap_suffix.isEmpty() && !cap_iconic.isEmpty())
-            // Keep the same suffix in iconic name if it's set
-            info->setVisibleIconName(QString(cap_iconic + cap_suffix).toUtf8());
-
-        if (isManaged() && decoration) {
-            decoration->captionChange();
-        }
-        emit captionChanged();
     }
+    QString shortcut_suffix = !shortcut().isEmpty() ? (" {" + shortcut().toString() + '}') : QString();
+    cap_suffix = machine_suffix + shortcut_suffix;
+    if ((!isSpecialWindow() || isToolbar()) && workspace()->findClient(FetchNameInternalPredicate(this))) {
+        int i = 2;
+        do {
+            cap_suffix = machine_suffix + " <" + QString::number(i) + '>' + LRM;
+            i++;
+        } while (workspace()->findClient(FetchNameInternalPredicate(this)));
+        info->setVisibleName(caption().toUtf8());
+        reset_name = false;
+    }
+    if ((was_suffix && cap_suffix.isEmpty()) || reset_name) {
+        // If it was new window, it may have old value still set, if the window is reused
+        info->setVisibleName("");
+        info->setVisibleIconName("");
+    } else if (!cap_suffix.isEmpty() && !cap_iconic.isEmpty())
+        // Keep the same suffix in iconic name if it's set
+        info->setVisibleIconName(QString(cap_iconic + cap_suffix).toUtf8());
+
+    if (isManaged() && decoration) {
+        decoration->captionChange();
+    }
+    emit captionChanged();
 }
 
 void Client::updateCaption()
@@ -1919,9 +1812,12 @@ void Client::fetchIconicName()
 /**
  * \reimp
  */
-QString Client::caption(bool full) const
+QString Client::caption(bool full, bool stripped) const
 {
-    return full ? cap_normal + cap_suffix : cap_normal;
+    QString cap = stripped ? cap_deco : cap_normal;
+    if (full)
+        cap += cap_suffix;
+    return cap;
 }
 
 bool Client::tabTo(Client *other, bool behind, bool activate)
@@ -2239,8 +2135,6 @@ void Client::getSyncCounter()
 void Client::sendSyncRequest()
 {
 #ifdef HAVE_XSYNC
-    delete m_readyForPaintingTimer;
-    m_readyForPaintingTimer = 0;
     if (syncRequest.counter == None || syncRequest.isPending)
         return; // do NOT, NEVER send a sync request when there's one on the stack. the clients will just stop respoding. FOREVER! ...
 
@@ -2372,8 +2266,7 @@ void Client::setBlockingCompositing(bool block)
     const bool usedToBlock = blocks_compositing;
     blocks_compositing = rules()->checkBlockCompositing(block);
     if (usedToBlock != blocks_compositing) {
-        workspace()->updateCompositeBlocking(blocks_compositing ? this : 0);
-        emit blockingCompositingChanged();
+        emit blockingCompositingChanged(blocks_compositing ? this : 0);
     }
 }
 
@@ -2522,6 +2415,60 @@ bool Client::isClient() const
     return true;
 }
 
+#ifdef KWIN_BUILD_KAPPMENU
+void Client::setAppMenuAvailable()
+{
+    m_menuAvailable = true;
+    emit appMenuAvailable();
+}
+
+void Client::setAppMenuUnavailable()
+{
+    m_menuAvailable = false;
+    emit appMenuUnavailable();
+}
+
+void Client::showApplicationMenu(const QPoint &p)
+{
+    workspace()->showApplicationMenu(p, window());
+}
+#endif
+
+NET::WindowType Client::windowType(bool direct, int supportedTypes) const
+{
+    // TODO: does it make sense to cache the returned window type for SUPPORTED_MANAGED_WINDOW_TYPES_MASK?
+    if (supportedTypes == 0) {
+        supportedTypes = SUPPORTED_MANAGED_WINDOW_TYPES_MASK;
+    }
+    NET::WindowType wt = info->windowType(supportedTypes);
+    if (direct) {
+        return wt;
+    }
+    NET::WindowType wt2 = client_rules.checkType(wt);
+    if (wt != wt2) {
+        wt = wt2;
+        info->setWindowType(wt);   // force hint change
+    }
+    // hacks here
+    if (wt == NET::Unknown)   // this is more or less suggested in NETWM spec
+        wt = isTransient() ? NET::Dialog : NET::Normal;
+    return wt;
+}
+
+bool Client::decorationHasAlpha() const
+{
+    if (!decoration || !workspace()->decorationHasAlpha()) {
+        // either no decoration or decoration has alpha disabled
+        return false;
+    }
+    if (workspace()->decorationSupportsAnnounceAlpha()) {
+        return decoration->isAlphaEnabled();
+    } else {
+        // decoration has alpha enabled and does not support alpha announcement
+        return true;
+    }
+}
+
 } // namespace
 
 #include "client.moc"
