@@ -24,6 +24,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <config-X11.h>
 
+#include "config-kwin.h"
+
 #include <QFrame>
 #include <QPixmap>
 #include <netwm.h>
@@ -274,6 +276,10 @@ class Client
      * Use with care!
      **/
     Q_PROPERTY(bool blocksCompositing READ isBlockingCompositing WRITE setBlockingCompositing NOTIFY blockingCompositingChanged)
+    /**
+     * Whether the decoration is currently using an alpha channel.
+     **/
+    Q_PROPERTY(bool decorationHasAlpha READ decorationHasAlpha)
 public:
     Client(Workspace* ws);
     Window wrapperId() const;
@@ -323,6 +329,7 @@ public:
 #ifdef HAVE_XSYNC
     void syncEvent(XSyncAlarmNotifyEvent* e);
 #endif
+    NET::WindowType windowType(bool direct = false, int supported_types = 0) const;
 
     bool manage(Window w, bool isMapped);
     void releaseWindow(bool on_shutdown = false);
@@ -491,7 +498,7 @@ public:
     inline bool isBlockingCompositing() { return blocks_compositing; }
     void updateCompositeBlocking(bool readProperty = false);
 
-    QString caption(bool full = true) const;
+    QString caption(bool full = true, bool stripped = false) const;
     void updateCaption();
 
     void keyPressEvent(uint key_code);   // FRAME ??
@@ -583,17 +590,8 @@ public:
     }
 
     // Decorations <-> Effects
-    const QPixmap *topDecoPixmap() const {
-        return &decorationPixmapTop;
-    }
-    const QPixmap *leftDecoPixmap() const {
-        return &decorationPixmapLeft;
-    }
-    const QPixmap *bottomDecoPixmap() const {
-        return &decorationPixmapBottom;
-    }
-    const QPixmap *rightDecoPixmap() const {
-        return &decorationPixmapRight;
+    PaintRedirector *decorationPaintRedirector() {
+        return paintRedirector;
     }
 
     int paddingLeft() const {
@@ -609,15 +607,14 @@ public:
         return padding_bottom;
     }
 
-    bool decorationPixmapRequiresRepaint();
-    void ensureDecorationPixmapsPainted();
-
     QRect decorationRect() const;
 
     QRect transparentRect() const;
 
     QRegion decorationPendingRegion() const;
 
+    bool decorationHasAlpha() const;
+
     enum CoordinateMode {
         DecorationRelative, // Relative to the top left corner of the decoration
         WindowRelative      // Relative to the top left corner of the window
@@ -639,6 +636,22 @@ public:
     void setSessionInteract(bool needed);
     virtual bool isClient() const;
 
+#ifdef KWIN_BUILD_KAPPMENU
+    // Used by workspace
+    void emitShowRequest() {
+        emit showRequest();
+    }
+    void emitMenuHidden() {
+        emit menuHidden();
+    }
+    void setAppMenuAvailable();
+    void setAppMenuUnavailable();
+    void showApplicationMenu(const QPoint&);
+    bool menuAvailable() {
+        return m_menuAvailable;
+    }
+#endif
+
 public slots:
     void closeWindow();
 
@@ -693,7 +706,6 @@ private slots:
     void removeSyncSupport();
     void pingTimeout();
     void demandAttentionKNotify();
-    void repaintDecorationPending();
 
     //Signals for the scripting interface
     //Signals make an excellent way for communication
@@ -729,6 +741,24 @@ signals:
      * another group, but not when a Client gets added or removed to the Client's ClientGroup.
      **/
     void tabGroupChanged();
+
+    /**
+     * Emitted whenever the Client want to show it menu
+     */
+    void showRequest();
+    /**
+     * Emitted whenever the Client's menu is closed
+     */
+    void menuHidden();
+    /**
+     * Emitted whenever the Client's menu is available
+     **/
+    void appMenuAvailable();
+    /**
+     * Emitted whenever the Client's menu is unavailable
+     */
+    void appMenuUnavailable();
+
     /**
      * Emitted whenever the demands attention state changes.
      **/
@@ -736,7 +766,7 @@ signals:
     /**
      * Emitted whenever the Client's block compositing state changes.
      **/
-    void blockingCompositingChanged();
+    void blockingCompositingChanged(KWin::Client *client);
 
 private:
     void exportMappingState(int s);   // ICCCM 4.1.3.1, 4.1.4, NETWM 2.5.1
@@ -794,8 +824,6 @@ private:
     void updateHiddenPreview();
 
     void updateInputShape();
-    void repaintDecorationPixmap(QPixmap& pix, const QRect& r, const QPixmap& src, QRegion reg);
-    void resizeDecorationPixmaps();
 
     Time readUserTimeMapTimestamp(const KStartupInfoId* asn_id, const KStartupInfoData* asn_data,
                                   bool session) const;
@@ -821,6 +849,7 @@ private:
     bool unrestrictedMoveResize;
     int moveResizeStartScreen;
     static bool s_haveResizeEffect;
+    bool m_managed;
 
     Position mode;
     QPoint moveOffset;
@@ -905,7 +934,7 @@ private:
     QTimer* shadeHoverTimer;
     QTimer* delayedMoveResizeTimer;
     Colormap cmap;
-    QString cap_normal, cap_iconic, cap_suffix;
+    QString cap_normal, cap_iconic, cap_suffix, cap_deco;
     Group* in_group;
     Window window_group;
     TabGroup* tab_group;
@@ -945,9 +974,6 @@ private:
     friend struct ResetupRulesProcedure;
     friend class GeometryUpdatesBlocker;
     QTimer* demandAttentionKNotifyTimer;
-    QPixmap decorationPixmapLeft, decorationPixmapRight, decorationPixmapTop, decorationPixmapBottom;
-    // we (instead of Qt) initialize the Pixmaps, and have to free them
-    bool m_responsibleForDecoPixmap;
     PaintRedirector* paintRedirector;
     QSharedPointer<TabBox::TabBoxClientImpl> m_tabBoxClient;
     bool m_firstInTabBox;
@@ -961,7 +987,11 @@ private:
     bool activitiesDefined; //whether the x property was actually set
 
     bool needsSessionInteract;
+    bool needsXWindowMove;
 
+#ifdef KWIN_BUILD_KAPPMENU
+    bool m_menuAvailable;
+#endif
     Window input_window;
     QPoint input_offset;
 };
@@ -1184,7 +1214,7 @@ inline int Client::sessionStackingOrder() const
 
 inline bool Client::isManaged() const
 {
-    return mapping_state != Withdrawn;
+    return m_managed;
 }
 
 inline QPoint Client::clientPos() const
