@@ -18,16 +18,15 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "cube.h"
+// KConfigSkeleton
+#include "cubeconfig.h"
+
 #include "cube_inside.h"
 
 #include <kaction.h>
 #include <kactioncollection.h>
 #include <klocale.h>
 #include <kwinconfig.h>
-#include <kconfiggroup.h>
-#include <kcolorscheme.h>
-#include <kglobal.h>
-#include <kstandarddirs.h>
 #include <kdebug.h>
 
 #include <QColor>
@@ -37,6 +36,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QKeyEvent>
 #include <QtConcurrentRun>
 #include <QVector2D>
+#include <QVector3D>
 
 #include <math.h>
 
@@ -115,7 +115,7 @@ CubeEffect::CubeEffect()
 
 bool CubeEffect::supported()
 {
-    return effects->compositingType() == OpenGLCompositing;
+    return effects->isOpenGLCompositing();
 }
 
 void CubeEffect::reconfigure(ReconfigureFlags)
@@ -125,7 +125,7 @@ void CubeEffect::reconfigure(ReconfigureFlags)
 
 void CubeEffect::loadConfig(QString config)
 {
-    KConfigGroup conf = effects->effectConfig(config);
+    CubeConfig::self()->readConfig();
     foreach (ElectricBorder border, borderActivate) {
         effects->unreserveElectricBorder(border);
     }
@@ -140,49 +140,47 @@ void CubeEffect::loadConfig(QString config)
     borderActivateSphere.clear();
     QList<int> borderList = QList<int>();
     borderList.append(int(ElectricNone));
-    borderList = conf.readEntry("BorderActivate", borderList);
+    borderList = CubeConfig::borderActivate();
     foreach (int i, borderList) {
         borderActivate.append(ElectricBorder(i));
         effects->reserveElectricBorder(ElectricBorder(i));
     }
     borderList.clear();
     borderList.append(int(ElectricNone));
-    borderList = conf.readEntry("BorderActivateCylinder", borderList);
+    borderList = CubeConfig::borderActivateCylinder();
     foreach (int i, borderList) {
         borderActivateCylinder.append(ElectricBorder(i));
         effects->reserveElectricBorder(ElectricBorder(i));
     }
     borderList.clear();
     borderList.append(int(ElectricNone));
-    borderList = conf.readEntry("BorderActivateSphere", borderList);
+    borderList = CubeConfig::borderActivateSphere();
     foreach (int i, borderList) {
         borderActivateSphere.append(ElectricBorder(i));
         effects->reserveElectricBorder(ElectricBorder(i));
     }
 
-    cubeOpacity = (float)conf.readEntry("Opacity", 80) / 100.0f;
-    opacityDesktopOnly = conf.readEntry("OpacityDesktopOnly", false);
-    displayDesktopName = conf.readEntry("DisplayDesktopName", true);
-    reflection = conf.readEntry("Reflection", true);
-    rotationDuration = animationTime(conf, "RotationDuration", 500);
-    backgroundColor = conf.readEntry("BackgroundColor", QColor(Qt::black));
-    capColor = conf.readEntry("CapColor", KColorScheme(QPalette::Active, KColorScheme::Window).background().color());
-    paintCaps = conf.readEntry("Caps", true);
-    closeOnMouseRelease = conf.readEntry("CloseOnMouseRelease", false);
-    float defaultZPosition = 100.0f;
-    if (config == "Sphere")
-        defaultZPosition = 450.0f;
-    zPosition = conf.readEntry("ZPosition", defaultZPosition);
-    useForTabBox = conf.readEntry("TabBox", false);
-    invertKeys = conf.readEntry("InvertKeys", false);
-    invertMouse = conf.readEntry("InvertMouse", false);
-    capDeformationFactor = conf.readEntry("CapDeformation", 0) / 100.0f;
-    useZOrdering = conf.readEntry("ZOrdering", false);
+    cubeOpacity = (float)CubeConfig::opacity() / 100.0f;
+    opacityDesktopOnly = CubeConfig::opacityDesktopOnly();
+    displayDesktopName = CubeConfig::displayDesktopName();
+    reflection = CubeConfig::reflection();
+    rotationDuration = animationTime(CubeConfig::rotationDuration() != 0 ? CubeConfig::rotationDuration() : 500);
+    backgroundColor = CubeConfig::backgroundColor();
+    capColor = CubeConfig::capColor();
+    paintCaps = CubeConfig::caps();
+    closeOnMouseRelease = CubeConfig::closeOnMouseRelease();
+    zPosition = CubeConfig::zPosition();
+
+    useForTabBox = CubeConfig::tabBox();
+    invertKeys = CubeConfig::invertKeys();
+    invertMouse = CubeConfig::invertMouse();
+    capDeformationFactor = (float)CubeConfig::capDeformation() / 100.0f;
+    useZOrdering = CubeConfig::zOrdering();
     delete wallpaper;
     wallpaper = NULL;
     delete capTexture;
     capTexture = NULL;
-    texturedCaps = conf.readEntry("TexturedCaps", true);
+    texturedCaps = CubeConfig::texturedCaps();
 
     timeLine.setCurveShape(QTimeLine::EaseInOutCurve);
     timeLine.setDuration(rotationDuration);
@@ -215,10 +213,9 @@ void CubeEffect::loadConfig(QString config)
     }
 
     // set the cap color on the shader
-    if (ShaderManager::instance()->isValid() && m_capShader->isValid()) {
-        ShaderManager::instance()->pushShader(m_capShader);
+    if (m_capShader->isValid()) {
+        ShaderBinder binder(m_capShader);
         m_capShader->setUniform("u_capColor", capColor);
-        ShaderManager::instance()->popShader();
     }
 }
 
@@ -296,7 +293,7 @@ void CubeEffect::slotWallPaperLoaded()
 bool CubeEffect::loadShader()
 {
     if (!(GLPlatform::instance()->supports(GLSL) &&
-            (effects->compositingType() == OpenGLCompositing)))
+            (effects->compositingType() == OpenGL2Compositing)))
         return false;
     QString fragmentshader       =  KGlobal::dirs()->findResource("data", "kwin/cylinder.frag");
     QString cylinderVertexshader =  KGlobal::dirs()->findResource("data", "kwin/cylinder.vert");
@@ -313,7 +310,7 @@ bool CubeEffect::loadShader()
         kError(1212) << "The cylinder shader failed to load!" << endl;
         return false;
     } else {
-        shaderManager->pushShader(cylinderShader);
+        ShaderBinder binder(cylinderShader);
         cylinderShader->setUniform("sampler", 0);
         QMatrix4x4 projection;
         float fovy = 60.0f;
@@ -336,7 +333,6 @@ bool CubeEffect::loadShader()
         cylinderShader->setUniform(GLShader::WindowTransformation, identity);
         QRect rect = effects->clientArea(FullArea, activeScreen, effects->currentDesktop());
         cylinderShader->setUniform("width", (float)rect.width() * 0.5f);
-        shaderManager->popShader();
     }
     // TODO: use generic shader - currently it is failing in alpha/brightness manipulation
     sphereShader = new GLShader(sphereVertexshader, fragmentshader);
@@ -344,7 +340,7 @@ bool CubeEffect::loadShader()
         kError(1212) << "The sphere shader failed to load!" << endl;
         return false;
     } else {
-        shaderManager->pushShader(sphereShader);
+        ShaderBinder binder(sphereShader);
         sphereShader->setUniform("sampler", 0);
         QMatrix4x4 projection;
         float fovy = 60.0f;
@@ -369,7 +365,6 @@ bool CubeEffect::loadShader()
         sphereShader->setUniform("width", (float)rect.width() * 0.5f);
         sphereShader->setUniform("height", (float)rect.height() * 0.5f);
         sphereShader->setUniform("u_offset", QVector2D(0, 0));
-        shaderManager->popShader();
         checkGLError("Loading Sphere Shader");
     }
     return true;
@@ -406,15 +401,10 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
 
         // wallpaper
         if (wallpaper) {
-            if (ShaderManager::instance()->isValid()) {
-                ShaderManager::instance()->pushShader(ShaderManager::SimpleShader);
-            }
+            ShaderBinder binder(ShaderManager::SimpleShader);
             wallpaper->bind();
             wallpaper->render(region, rect);
             wallpaper->unbind();
-            if (ShaderManager::instance()->isValid()) {
-                ShaderManager::instance()->popShader();
-            }
         }
 
         glEnable(GL_BLEND);
@@ -461,7 +451,7 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
             popMatrix();
 
             // call the inside cube effects
-#ifndef KWIN_HAVE_OPENGLES
+#ifdef KWIN_HAVE_OPENGL_1
             foreach (CubeInsideEffect * inside, m_cubeInsideEffects) {
                 pushMatrix(m_rotationMatrix);
                 glTranslatef(rect.width() / 2, rect.height() / 2, -point - zTranslate);
@@ -502,7 +492,7 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
             ShaderManager *shaderManager = ShaderManager::instance();
             if (shaderManager->isValid() && m_reflectionShader->isValid()) {
                 // ensure blending is enabled - no attribute stack
-                shaderManager->pushShader(m_reflectionShader);
+                ShaderBinder binder(m_reflectionShader);
                 QMatrix4x4 windowTransformation;
                 windowTransformation.translate(rect.x() + rect.width() * 0.5f, 0.0, 0.0);
                 m_reflectionShader->setUniform("windowTransformation", windowTransformation);
@@ -527,10 +517,8 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
                 vbo->reset();
                 vbo->setData(6, 3, verts.data(), texcoords.data());
                 vbo->render(GL_TRIANGLES);
-
-                shaderManager->popShader();
             } else {
-#ifndef KWIN_HAVE_OPENGLES
+#ifdef KWIN_HAVE_OPENGL_1
                 glColor4f(0.0, 0.0, 0.0, alpha);
                 glPushMatrix();
                 glTranslatef(rect.x() + rect.width() * 0.5f, 0.0, 0.0);
@@ -560,7 +548,7 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
 
 
         // call the inside cube effects
-#ifndef KWIN_HAVE_OPENGLES
+#ifdef KWIN_HAVE_OPENGL_1
         foreach (CubeInsideEffect * inside, m_cubeInsideEffects) {
             pushMatrix(m_rotationMatrix);
             glTranslatef(rect.width() / 2, rect.height() / 2, -point - zTranslate);
@@ -601,7 +589,7 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
         }
         // restore the ScreenTransformation after all desktops are painted
         // if not done GenericShader keeps the rotation data and transforms windows incorrectly in other rendering calls
-        if (ShaderManager::instance()->isValid()) {
+        if (effects->compositingType() == OpenGL2Compositing) {
             GLShader *shader = ShaderManager::instance()->pushShader(KWin::ShaderManager::GenericShader);
             shader->setUniform(GLShader::ScreenTransformation, QMatrix4x4());
             ShaderManager::instance()->popShader();
@@ -743,13 +731,10 @@ void CubeEffect::paintCube(int mask, QRegion region, ScreenPaintData& data)
             painting_desktop = effects->numberOfDesktops();
         }
         ScreenPaintData newData = data;
-        RotationData rot = RotationData();
-        rot.axis = RotationData::YAxis;
-        rot.angle = internalCubeAngle * i;
-        rot.xRotationPoint = rect.width() / 2;
-        rot.zRotationPoint = -point;
-        newData.rotation = &rot;
-        newData.zTranslate = -zTranslate;
+        newData.setRotationAxis(Qt::YAxis);
+        newData.setRotationAngle(internalCubeAngle * i);
+        newData.setRotationOrigin(QVector3D(rect.width() / 2, 0.0, -point));
+        newData.setZTranslation(-zTranslate);
         effects->paintScreen(mask, region, newData);
     }
     cube_painting = false;
@@ -791,7 +776,7 @@ void CubeEffect::paintCap(bool frontFirst, float zOffset)
     }
 
     bool capShader = false;
-    if (ShaderManager::instance()->isValid() && m_capShader->isValid()) {
+    if (effects->compositingType() == OpenGL2Compositing && m_capShader->isValid()) {
         capShader = true;
         ShaderManager::instance()->pushShader(m_capShader);
         float opacity = cubeOpacity;
@@ -813,14 +798,12 @@ void CubeEffect::paintCap(bool frontFirst, float zOffset)
             capTexture->bind();
         }
     } else {
+#ifdef KWIN_HAVE_OPENGL_1
         pushMatrix(m_rotationMatrix * capMatrix);
 
-#ifndef KWIN_HAVE_OPENGLES
         glMatrixMode(GL_TEXTURE);
-#endif
         pushMatrix();
         loadMatrix(m_textureMirrorMatrix);
-#ifndef KWIN_HAVE_OPENGLES
         glMatrixMode(GL_MODELVIEW);
 
         glColor4f(capColor.redF(), capColor.greenF(), capColor.blueF(), cubeOpacity);
@@ -1358,7 +1341,7 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
                 zOrdering *= timeLine.currentValue();
             if (stop)
                 zOrdering *= (1.0 - timeLine.currentValue());
-            data.zTranslate += zOrdering;
+            data.translate(0.0, 0.0, zOrdering);
         }
         // check for windows belonging to the previous desktop
         int prev_desktop = painting_desktop - 1;
@@ -1380,13 +1363,11 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
             }
             data.quads = new_quads;
             if (shader) {
-                data.xTranslate = -rect.width();
+                data.setXTranslation(-rect.width());
             } else {
-                RotationData rot = RotationData();
-                rot.axis = RotationData::YAxis;
-                rot.xRotationPoint = rect.width() - w->x();
-                rot.angle = 360.0f / effects->numberOfDesktops();
-                data.rotation = &rot;
+                data.setRotationAxis(Qt::YAxis);
+                data.setRotationOrigin(QVector3D(rect.width() - w->x(), 0.0, 0.0));
+                data.setRotationAngle(-360.0f / effects->numberOfDesktops());
                 float cubeAngle = (float)((float)(effects->numberOfDesktops() - 2) / (float)effects->numberOfDesktops() * 180.0f);
                 float point = rect.width() / 2 * tan(cubeAngle * 0.5f * M_PI / 180.0f);
                 QMatrix4x4 matrix;
@@ -1406,13 +1387,11 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
             }
             data.quads = new_quads;
             if (shader) {
-                data.xTranslate = rect.width();
+                data.setXTranslation(rect.width());
             } else {
-                RotationData rot = RotationData();
-                rot.axis = RotationData::YAxis;
-                rot.xRotationPoint = -w->x();
-                rot.angle = -360.0f / effects->numberOfDesktops();
-                data.rotation = &rot;
+                data.setRotationAxis(Qt::YAxis);
+                data.setRotationOrigin(QVector3D(-w->x(), 0.0, 0.0));
+                data.setRotationAngle(-360.0f / effects->numberOfDesktops());
                 float cubeAngle = (float)((float)(effects->numberOfDesktops() - 2) / (float)effects->numberOfDesktops() * 180.0f);
                 float point = rect.width() / 2 * tan(cubeAngle * 0.5f * M_PI / 180.0f);
                 QMatrix4x4 matrix;
@@ -1445,7 +1424,7 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
             opacity = 0.99f;
         if (opacityDesktopOnly && !w->isDesktop())
             opacity = 0.99f;
-        data.opacity *= opacity;
+        data.multiplyOpacity(opacity);
 
         if (w->isOnDesktop(painting_desktop) && w->x() < rect.x()) {
             WindowQuadList new_quads;
@@ -1569,7 +1548,7 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
                     }
                 }
                 bool capShader = false;
-                if (ShaderManager::instance()->isValid() && m_capShader->isValid()) {
+                if (effects->compositingType() == OpenGL2Compositing && m_capShader->isValid()) {
                     capShader = true;
                     ShaderManager::instance()->pushShader(m_capShader);
                     m_capShader->setUniform("u_mirror", 0);
@@ -1908,14 +1887,13 @@ void CubeEffect::setActive(bool active)
         inside->setActive(true);
     }
     if (active) {
-        KConfigGroup conf = effects->effectConfig("Cube");
-        QString capPath = conf.readEntry("CapPath", KGlobal::dirs()->findResource("appdata", "cubecap.png"));
+        QString capPath = CubeConfig::capPath();
         if (texturedCaps && !capTexture && !capPath.isEmpty()) {
             QFutureWatcher<QImage> *watcher = new QFutureWatcher<QImage>(this);
             connect(watcher, SIGNAL(finished()), SLOT(slotCubeCapLoaded()));
             watcher->setFuture(QtConcurrent::run(this, &CubeEffect::loadCubeCap, capPath));
         }
-        QString wallpaperPath = conf.readEntry("Wallpaper", QString(""));
+        QString wallpaperPath = CubeConfig::wallpaper().toLocalFile();
         if (!wallpaper && !wallpaperPath.isEmpty()) {
             QFutureWatcher<QImage> *watcher = new QFutureWatcher<QImage>(this);
             connect(watcher, SIGNAL(finished()), SLOT(slotWallPaperLoaded()));
