@@ -20,6 +20,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
 #include "zoom.h"
+// KConfigSkeleton
+#include "zoomconfig.h"
 
 #include <QtCore/QDir>
 #include <QtCore/QFile>
@@ -28,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtGui/QX11Info>
 #include <QtGui/QApplication>
 #include <QtGui/QStyle>
+#include <QtGui/QVector2D>
 #include <QtDBus/QDBusConnection>
 #include <kaction.h>
 #include <kactioncollection.h>
@@ -151,8 +154,6 @@ void ZoomEffect::showCursor()
 
 void ZoomEffect::hideCursor()
 {
-    if (mouseTracking == MouseTrackingProportional && mousePointer == MousePointerKeep)
-        return; // don't replace the actual cursor by a static image for no reason.
     if (!isMouseHidden) {
         // try to load the cursor-theme into a OpenGL texture and if successful then hide the mouse-pointer
         recreateTexture();
@@ -187,7 +188,7 @@ void ZoomEffect::recreateTexture()
         imageWidth = ximg->width;
         imageHeight = ximg->height;
         QImage img((uchar*)ximg->pixels, imageWidth, imageHeight, QImage::Format_ARGB32_Premultiplied);
-        if (effects->compositingType() == OpenGLCompositing)
+        if (effects->isOpenGLCompositing())
             texture = new GLTexture(img);
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
         if (effects->compositingType() == XRenderCompositing)
@@ -203,15 +204,15 @@ void ZoomEffect::recreateTexture()
 
 void ZoomEffect::reconfigure(ReconfigureFlags)
 {
-    KConfigGroup conf = EffectsHandler::effectConfig("Zoom");
+    ZoomConfig::self()->readConfig();
     // On zoom-in and zoom-out change the zoom by the defined zoom-factor.
-    zoomFactor = qMax(0.1, conf.readEntry("ZoomFactor", zoomFactor));
+    zoomFactor = qMax(0.1, ZoomConfig::zoomFactor());
     // Visibility of the mouse-pointer.
-    mousePointer = MousePointerType(conf.readEntry("MousePointer", int(mousePointer)));
+    mousePointer = MousePointerType(ZoomConfig::mousePointer());
     // Track moving of the mouse.
-    mouseTracking = MouseTrackingType(conf.readEntry("MouseTracking", int(mouseTracking)));
+    mouseTracking = MouseTrackingType(ZoomConfig::mouseTracking());
     // Enable tracking of the focused location.
-    bool _enableFocusTracking = conf.readEntry("EnableFocusTracking", enableFocusTracking);
+    bool _enableFocusTracking = ZoomConfig::enableFocusTracking();
     if (enableFocusTracking != _enableFocusTracking) {
         enableFocusTracking = _enableFocusTracking;
         if (QDBusConnection::sessionBus().isConnected()) {
@@ -222,13 +223,13 @@ void ZoomEffect::reconfigure(ReconfigureFlags)
         }
     }
     // When the focus changes, move the zoom area to the focused location.
-    followFocus = conf.readEntry("EnableFollowFocus", followFocus);
+    followFocus = ZoomConfig::enableFollowFocus();
     // The time in milliseconds to wait before a focus-event takes away a mouse-move.
-    focusDelay = qMax(0, conf.readEntry("FocusDelay", focusDelay));
+    focusDelay = qMax(uint(0), ZoomConfig::focusDelay());
     // The factor the zoom-area will be moved on touching an edge on push-mode or using the navigation KAction's.
-    moveFactor = qMax(0.1, conf.readEntry("MoveFactor", moveFactor));
+    moveFactor = qMax(0.1, ZoomConfig::moveFactor());
     // Load the saved zoom value.
-    target_zoom = conf.readEntry("InitialZoom", target_zoom);
+    target_zoom = ZoomConfig::initialZoom();
     if (target_zoom > 1.0)
         zoomIn(target_zoom);
 }
@@ -265,22 +266,21 @@ static XTransform xrenderIdentity = {{
 void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
 {
     if (zoom != 1.0) {
-        data.xScale *= zoom;
-        data.yScale *= zoom;
+        data *= QVector2D(zoom, zoom);
 
         // mouse-tracking allows navigation of the zoom-area using the mouse.
         switch(mouseTracking) {
         case MouseTrackingProportional:
-            data.xTranslate = - int(cursorPoint.x() * (zoom - 1.0));
-            data.yTranslate = - int(cursorPoint.y() * (zoom - 1.0));
+            data.setXTranslation(- int(cursorPoint.x() * (zoom - 1.0)));
+            data.setYTranslation(- int(cursorPoint.y() * (zoom - 1.0)));
             prevPoint = cursorPoint;
             break;
         case MouseTrackingCentred:
             prevPoint = cursorPoint;
             // fall through
         case MouseTrackingDisabled:
-            data.xTranslate = qMin(0, qMax(int(displayWidth() - displayWidth() * zoom), int(displayWidth() / 2 - prevPoint.x() * zoom)));
-            data.yTranslate = qMin(0, qMax(int(displayHeight() - displayHeight() * zoom), int(displayHeight() / 2 - prevPoint.y() * zoom)));
+            data.setXTranslation(qMin(0, qMax(int(displayWidth() - displayWidth() * zoom), int(displayWidth() / 2 - prevPoint.x() * zoom))));
+            data.setYTranslation(qMin(0, qMax(int(displayHeight() - displayHeight() * zoom), int(displayHeight() / 2 - prevPoint.y() * zoom))));
             break;
         case MouseTrackingPush:
             if (timeline.state() != QTimeLine::Running) {
@@ -303,8 +303,8 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
                     timeline.start();
                 }
             }
-            data.xTranslate = - int(prevPoint.x() * (zoom - 1.0));
-            data.yTranslate = - int(prevPoint.y() * (zoom - 1.0));
+            data.setXTranslation(- int(prevPoint.x() * (zoom - 1.0)));
+            data.setYTranslation(- int(prevPoint.y() * (zoom - 1.0)));
             break;
         }
 
@@ -318,8 +318,8 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
                 acceptFocus = msecs > focusDelay;
             }
             if (acceptFocus) {
-                data.xTranslate = - int(focusPoint.x() * (zoom - 1.0));
-                data.yTranslate = - int(focusPoint.y() * (zoom - 1.0));
+                data.setXTranslation(- int(focusPoint.x() * (zoom - 1.0)));
+                data.setYTranslation(- int(focusPoint.y() * (zoom - 1.0)));
                 prevPoint = focusPoint;
             }
         }
@@ -338,7 +338,7 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
             h *= zoom;
         }
         QPoint p = QCursor::pos();
-        QRect rect(p.x() * zoom + data.xTranslate, p.y() * zoom + data.yTranslate, w, h);
+        QRect rect(p.x() * zoom + data.xTranslation(), p.y() * zoom + data.yTranslation(), w, h);
 
         if (texture) {
             texture->bind();
