@@ -43,6 +43,7 @@
 #include <QtGui/QSortFilterProxyModel>
 #include <QtGui/QGraphicsObject>
 #include <QtGui/QScrollBar>
+#include <QUiLoader>
 // KDE
 #include <KAboutData>
 #include <KDialog>
@@ -50,7 +51,9 @@
 #include <KMessageBox>
 #include <KNS3/DownloadDialog>
 #include <KDE/KStandardDirs>
+#include <KDE/KConfigDialogManager>
 #include <KPluginFactory>
+#include <Plasma/ConfigLoader>
 #include <qdeclarative.h>
 
 // KCModule plugin interface
@@ -108,10 +111,15 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
         m_ui->decorationList->engine()->addImportPath(importPath);
     }
     m_ui->decorationList->rootContext()->setContextProperty("decorationModel", m_proxyModel);
+    m_ui->decorationList->rootContext()->setContextProperty("decorationBaseModel", m_model);
     m_ui->decorationList->rootContext()->setContextProperty("options", m_decorationButtons);
     m_ui->decorationList->rootContext()->setContextProperty("highlightColor", m_ui->decorationList->palette().color(QPalette::Highlight));
     m_ui->decorationList->rootContext()->setContextProperty("sliderWidth", m_ui->decorationList->verticalScrollBar()->width());
     m_ui->decorationList->rootContext()->setContextProperty("auroraeSource", KStandardDirs::locate("data", "kwin/aurorae/aurorae.qml"));
+    m_ui->decorationList->rootContext()->setContextProperty("decorationActiveCaptionColor", KDecoration::options()->color(ColorFont, true));
+    m_ui->decorationList->rootContext()->setContextProperty("decorationInactiveCaptionColor", KDecoration::options()->color(ColorFont, false));
+    m_ui->decorationList->rootContext()->setContextProperty("decorationActiveTitleBarColor", KDecoration::options()->color(ColorTitleBar, true));
+    m_ui->decorationList->rootContext()->setContextProperty("decorationInactiveTitleBarColor", KDecoration::options()->color(ColorTitleBar, false));
     m_ui->decorationList->setSource(mainQmlPath);
 
     readConfig(style);
@@ -168,12 +176,11 @@ void KWinDecorationModule::readConfig(const KConfigGroup & conf)
     // Find the corresponding decoration name to that of
     // the current plugin library name
 
-    QString libraryName = conf.readEntry("PluginLib",
-                                         ((QPixmap::defaultDepth() > 8) ? "kwin3_oxygen" : "kwin3_plastik"));
+    QString libraryName = conf.readEntry("PluginLib", "kwin3_oxygen");
 
     if (libraryName.isEmpty()) {
         // Selected decoration doesn't exist, use the default
-        libraryName = ((QPixmap::defaultDepth() > 8) ? "kwin3_oxygen" : "kwin3_plastik");
+        libraryName = "kwin3_oxygen";
     }
 
     const int bsize = conf.readEntry("BorderSize", (int)BorderNormal);
@@ -184,7 +191,8 @@ void KWinDecorationModule::readConfig(const KConfigGroup & conf)
         KConfig auroraeConfig("auroraerc");
         KConfigGroup group(&auroraeConfig, "Engine");
         const QString themeName = group.readEntry("ThemeName", "example-deco");
-        const QModelIndex index = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(themeName));
+        const QString type = group.readEntry("EngineType", "aurorae");
+        const QModelIndex index = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(themeName, type));
         if (index.isValid()) {
             m_ui->decorationList->rootObject()->setProperty("currentIndex", index.row());
         }
@@ -231,10 +239,16 @@ void KWinDecorationModule::writeConfig(KConfigGroup & conf)
     conf.writeEntry("BorderSize",
                     static_cast<int>(m_model->data(index, DecorationModel::BorderSizeRole).toInt()));
 
-    if (m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::AuroraeDecoration) {
+    if (m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::AuroraeDecoration ||
+        m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration) {
         KConfig auroraeConfig("auroraerc");
         KConfigGroup group(&auroraeConfig, "Engine");
         group.writeEntry("ThemeName", m_model->data(index, DecorationModel::AuroraeNameRole).toString());
+        if (m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration) {
+            group.writeEntry("EngineType", "qml");
+        } else {
+            group.deleteEntry("EngineType");
+        }
         group.sync();
     }
 
@@ -316,10 +330,15 @@ void KWinDecorationModule::slotGHNSClicked()
             const QModelIndex index = m_proxyModel->mapToSource(m_proxyModel->index(m_ui->decorationList->rootObject()->property("currentIndex").toInt(), 0));
             const QString libraryName = m_model->data(index, DecorationModel::LibraryNameRole).toString();
             bool aurorae = m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::AuroraeDecoration;
+            bool qml = m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration;
             const QString auroraeName = m_model->data(index, DecorationModel::AuroraeNameRole).toString();
             m_model->reload();
             if (aurorae) {
-                const QModelIndex proxyIndex = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(auroraeName));
+                const QModelIndex proxyIndex = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(auroraeName, "aurorae"));
+                if (proxyIndex.isValid())
+                    m_ui->decorationList->rootObject()->setProperty("currentIndex", proxyIndex.row());
+            } else if (qml) {
+                const QModelIndex proxyIndex = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(auroraeName, "qml"));
                 if (proxyIndex.isValid())
                     m_ui->decorationList->rootObject()->setProperty("currentIndex", proxyIndex.row());
             } else {
@@ -336,7 +355,8 @@ void KWinDecorationModule::slotConfigureDecoration()
 {
     const QModelIndex index = m_proxyModel->mapToSource(m_proxyModel->index(m_ui->decorationList->rootObject()->property("currentIndex").toInt(), 0));
     bool reload = false;
-    if (index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::AuroraeDecoration) {
+    if (index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::AuroraeDecoration ||
+        index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration) {
         QPointer< KDialog > dlg = new KDialog(this);
         dlg->setCaption(i18n("Decoration Options"));
         dlg->setButtons(KDialog::Ok | KDialog::Cancel);
@@ -344,9 +364,40 @@ void KWinDecorationModule::slotConfigureDecoration()
         dlg->setMainWidget(form);
         form->borderSizesCombo->setCurrentIndex(index.data(DecorationModel::BorderSizeRole).toInt());
         form->buttonSizesCombo->setCurrentIndex(index.data(DecorationModel::ButtonSizeRole).toInt());
+        form->closeWindowsDoubleClick->setChecked(index.data(DecorationModel::CloseOnDblClickRole).toBool());
+        // in case of QmlDecoration look for a config.ui in the package structure
+        KConfigDialogManager *configManager = NULL;
+        if (index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration) {
+            const QString packageName = index.data(DecorationModel::AuroraeNameRole).toString();
+            const QString uiPath = KStandardDirs::locate("data", "kwin/decorations/" + packageName + "/contents/ui/config.ui");
+            const QString configPath = KStandardDirs::locate("data", "kwin/decorations/" + packageName + "/contents/config/main.xml");
+            if (!uiPath.isEmpty() && !configPath.isEmpty()) {
+                // load the KConfigSkeleton
+                QFile configFile(configPath);
+                KSharedConfigPtr auroraeConfig = KSharedConfig::openConfig("auroraerc");
+                KConfigGroup configGroup = auroraeConfig->group(packageName);
+                Plasma::ConfigLoader *skeleton = new Plasma::ConfigLoader(&configGroup, &configFile, dlg);
+                // load the ui file
+                QUiLoader *loader = new QUiLoader(dlg);
+                QFile uiFile(uiPath);
+                uiFile.open(QFile::ReadOnly);
+                QWidget *customConfigForm = loader->load(&uiFile, form);
+                uiFile.close();
+                form->layout()->addWidget(customConfigForm);
+                // connect the ui file with the skeleton
+                configManager = new KConfigDialogManager(customConfigForm, skeleton);
+                configManager->updateWidgets();
+            }
+        }
         if (dlg->exec() == KDialog::Accepted) {
             m_model->setData(index, form->borderSizesCombo->currentIndex(), DecorationModel::BorderSizeRole);
             m_model->setData(index, form->buttonSizesCombo->currentIndex(), DecorationModel::ButtonSizeRole);
+            m_model->setData(index, form->closeWindowsDoubleClick->isChecked(), DecorationModel::CloseOnDblClickRole);
+            if (configManager && configManager->hasChanged()) {
+                // we have a config manager and the settings changed
+                configManager->updateSettings();
+                m_model->notifyConfigChanged(index);
+            }
             reload = true;
         }
         delete dlg;
