@@ -29,6 +29,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtCore/QPair>
 #include <QtCore/QSet>
 #include <QtCore/QRect>
+#include <QtGui/QGraphicsRotation>
 #include <QtGui/QRegion>
 
 #include <QtCore/QVector>
@@ -48,10 +49,14 @@ class KConfigGroup;
 class KActionCollection;
 class QFont;
 class QKeyEvent;
+class QVector2D;
 
 namespace KWin
 {
 
+class PaintDataPrivate;
+class WindowPaintDataPrivate;
+
 class EffectWindow;
 class EffectWindowGroup;
 class EffectFrame;
@@ -60,7 +65,6 @@ class Effect;
 class WindowQuad;
 class GLShader;
 class XRenderPicture;
-class RotationData;
 class WindowQuadList;
 class WindowPrePaintData;
 class WindowPaintData;
@@ -779,6 +783,12 @@ public:
     Q_SCRIPTABLE virtual void addRepaint(int x, int y, int w, int h) = 0;
 
     CompositingType compositingType() const;
+    /**
+     * @brief Whether the Compositor is OpenGL based (either GL 1 or 2).
+     *
+     * @return bool @c true in case of OpenGL based Compositor, @c false otherwise
+     **/
+    bool isOpenGLCompositing() const;
     virtual unsigned long xrenderBufferPicture() = 0;
     virtual void reconfigure() = 0;
 
@@ -991,7 +1001,7 @@ Q_SIGNALS:
      * Signal emitted when an area of a window is scheduled for repainting.
      * Use this signal in an effect if another area needs to be synced as well.
      * @param w The window which is scheduled for repainting
-     * @param r The damaged rect
+     * @param r Always empty.
      * @since 4.7
      **/
     void windowDamaged(KWin::EffectWindow *w, const QRect &r);
@@ -1310,6 +1320,11 @@ class KWIN_EXPORT EffectWindow : public QObject
     Q_PROPERTY(QStringList activities READ activities)
     Q_PROPERTY(bool onCurrentActivity READ isOnCurrentActivity)
     Q_PROPERTY(bool onAllActivities READ isOnAllActivities)
+    /**
+     * Whether the decoration currently uses an alpha channel.
+     * @since 4.10
+     **/
+    Q_PROPERTY(bool decorationHasAlpha READ decorationHasAlpha)
 public:
     /**  Flags explaining why painting should be disabled  */
     enum {
@@ -1397,6 +1412,7 @@ public:
      */
     virtual QRect decorationInnerRect() const = 0;
     bool hasDecoration() const;
+    bool decorationHasAlpha() const;
     virtual QByteArray readProperty(long atom, long type, int format) const = 0;
     virtual void deleteProperty(long atom) const = 0;
 
@@ -1632,7 +1648,7 @@ public:
      **/
     QRegion paint;
     /**
-     * The clip region will be substracted from paint region of following windows.
+     * The clip region will be subtracted from paint region of following windows.
      * I.e. window will definitely cover it's clip region
      **/
     QRegion clip;
@@ -1648,24 +1664,246 @@ public:
     void setTransformed();
 };
 
-class KWIN_EXPORT WindowPaintData
+class KWIN_EXPORT PaintData
+{
+public:
+    virtual ~PaintData();
+    /**
+     * @returns scale factor in X direction.
+     * @since 4.10
+     **/
+    qreal xScale() const;
+    /**
+     * @returns scale factor in Y direction.
+     * @since 4.10
+     **/
+    qreal yScale() const;
+    /**
+     * @returns scale factor in Z direction.
+     * @since 4.10
+     **/
+    qreal zScale() const;
+    /**
+     * Sets the scale factor in X direction to @p scale
+     * @param scale The scale factor in X direction
+     * @since 4.10
+     **/
+    void setXScale(qreal scale);
+    /**
+     * Sets the scale factor in Y direction to @p scale
+     * @param scale The scale factor in Y direction
+     * @since 4.10
+     **/
+    void setYScale(qreal scale);
+    /**
+     * Sets the scale factor in Z direction to @p scale
+     * @param scale The scale factor in Z direction
+     * @since 4.10
+     **/
+    void setZScale(qreal scale);
+    /**
+     * Sets the scale factor in X and Y direction.
+     * @param scale The scale factor for X and Y direction
+     * @since 4.10
+     **/
+    void setScale(const QVector2D &scale);
+    /**
+     * Sets the scale factor in X, Y and Z direction
+     * @param scale The scale factor for X, Y and Z direction
+     * @since 4.10
+     **/
+    void setScale(const QVector3D &scale);
+    const QGraphicsScale &scale() const;
+    const QVector3D &translation() const;
+    /**
+     * @returns the translation in X direction.
+     * @since 4.10
+     **/
+    qreal xTranslation() const;
+    /**
+     * @returns the translation in Y direction.
+     * @since 4.10
+     **/
+    qreal yTranslation() const;
+    /**
+     * @returns the translation in Z direction.
+     * @since 4.10
+     **/
+    qreal zTranslation() const;
+    /**
+     * Sets the translation in X direction to @p translate.
+     * @since 4.10
+     **/
+    void setXTranslation(qreal translate);
+    /**
+     * Sets the translation in Y direction to @p translate.
+     * @since 4.10
+     **/
+    void setYTranslation(qreal translate);
+    /**
+     * Sets the translation in Z direction to @p translate.
+     * @since 4.10
+     **/
+    void setZTranslation(qreal translate);
+    /**
+     * Performs a translation by adding the values component wise.
+     * @param x Translation in X direction
+     * @param y Translation in Y direction
+     * @param z Translation in Z direction
+     * @since 4.10
+     **/
+    void translate(qreal x, qreal y = 0.0, qreal z = 0.0);
+    /**
+     * Performs a translation by adding the values component wise.
+     * Overloaded method for convenience.
+     * @param translate The translation
+     * @since 4.10
+     **/
+    void translate(const QVector3D &translate);
+
+    /**
+     * Sets the rotation angle.
+     * @param angle The new rotation angle.
+     * @since 4.10
+     * @see rotationAngle()
+     **/
+    void setRotationAngle(qreal angle);
+    /**
+     * Returns the rotation angle.
+     * Initially 0.0.
+     * @returns The current rotation angle.
+     * @since 4.10
+     * @see setRotationAngle
+     **/
+    qreal rotationAngle() const;
+    /**
+     * Sets the rotation origin.
+     * @param origin The new rotation origin.
+     * @since 4.10
+     * @see rotationOrigin()
+     **/
+    void setRotationOrigin(const QVector3D &origin);
+    /**
+     * Returns the rotation origin. That is the point in space which is fixed during the rotation.
+     * Initially this is 0/0/0.
+     * @returns The rotation's origin
+     * @since 4.10
+     * @see setRotationOrigin()
+     **/
+    QVector3D rotationOrigin() const;
+    /**
+     * Sets the rotation axis.
+     * Set a component to 1.0 to rotate around this axis and to 0.0 to disable rotation around the
+     * axis.
+     * @param axis A vector holding information on which axis to rotate
+     * @since 4.10
+     * @see rotationAxis()
+     **/
+    void setRotationAxis(const QVector3D &axis);
+    /**
+     * Sets the rotation axis.
+     * Overloaded method for convenience.
+     * @param axis The axis around which should be rotated.
+     * @since 4.10
+     * @see rotationAxis()
+     **/
+    void setRotationAxis(Qt::Axis axis);
+    /**
+     * The current rotation axis.
+     * By default the rotation is (0/0/1) which means a rotation around the z axis.
+     * @returns The current rotation axis.
+     * @since 4.10
+     * @see setRotationAxis
+     **/
+    QVector3D rotationAxis() const;
+
+protected:
+    PaintData();
+    PaintData(const PaintData &other);
+
+private:
+    PaintDataPrivate * const d;
+};
+
+class KWIN_EXPORT WindowPaintData : public PaintData
 {
 public:
     WindowPaintData(EffectWindow* w);
+    WindowPaintData(const WindowPaintData &other);
+    virtual ~WindowPaintData();
+    /**
+     * Scales the window by @p scale factor.
+     * Multiplies all three components by the given factor.
+     * @since 4.10
+     **/
+    WindowPaintData& operator*=(qreal scale);
+    /**
+     * Scales the window by @p scale factor.
+     * Performs a component wise multiplication on x and y components.
+     * @since 4.10
+     **/
+    WindowPaintData& operator*=(const QVector2D &scale);
+    /**
+     * Scales the window by @p scale factor.
+     * Performs a component wise multiplication.
+     * @since 4.10
+     **/
+    WindowPaintData& operator*=(const QVector3D &scale);
+    /**
+     * Translates the window by the given @p translation and returns a reference to the ScreenPaintData.
+     * @since 4.10
+     **/
+    WindowPaintData& operator+=(const QPointF &translation);
+    /**
+     * Translates the window by the given @p translation and returns a reference to the ScreenPaintData.
+     * Overloaded method for convenience.
+     * @since 4.10
+     **/
+    WindowPaintData& operator+=(const QPoint &translation);
+    /**
+     * Translates the window by the given @p translation and returns a reference to the ScreenPaintData.
+     * Overloaded method for convenience.
+     * @since 4.10
+     **/
+    WindowPaintData& operator+=(const QVector2D &translation);
+    /**
+     * Translates the window by the given @p translation and returns a reference to the ScreenPaintData.
+     * Overloaded method for convenience.
+     * @since 4.10
+     **/
+    WindowPaintData& operator+=(const QVector3D &translation);
     /**
      * Window opacity, in range 0 = transparent to 1 = fully opaque
-     * Opacity for contents is opacity*contents_opacity, the same
-     * way for decoration.
+     * Opacity for decoration is opacity*decorationOpacity
+     * @see decorationOpacity
+     * @see setOpacity
+     * @see setDecorationOpacity
+     * @since 4.10
      */
-    double opacity;
-    double contents_opacity;
-    double decoration_opacity;
-    double xScale;
-    double yScale;
-    double zScale;
-    int xTranslate;
-    int yTranslate;
-    double zTranslate;
+    qreal opacity() const;
+    qreal decorationOpacity() const;
+    /**
+     * Sets the window opacity to the new @p opacity.
+     * If you want to modify the existing opacity level consider using multiplyOpacity.
+     * @param opacity The new opacity level
+     * @since 4.10
+     **/
+    void setOpacity(qreal opacity);
+    void setDecorationOpacity(qreal opacity);
+    /**
+     * Multiplies the current opacity with the @p factor.
+     * @param factor Factor with which the opacity should be multiplied
+     * @return New opacity level
+     * @since 4.10
+     **/
+    qreal multiplyOpacity(qreal factor);
+    /**
+     * Multiplies the current decoration opacity with the @p factor.
+     * @param factor Factor with which the opacity should be multiplied
+     * @return New decoration opacity level
+     * @since 4.10
+     **/
+    qreal multiplyDecorationOpacity(qreal factor);
     /**
      * Saturation of the window, in range [0; 1]
      * 1 means that the window is unchanged, 0 means that it's completely
@@ -1673,33 +1911,112 @@ public:
      *  but not completely grey
      * Use EffectsHandler::saturationSupported() to find out whether saturation
      * is supported by the system, otherwise this value has no effect.
+     * @return The current saturation
+     * @see setSaturation()
+     * @since 4.10
      **/
-    double saturation;
+    qreal saturation() const;
+    /**
+     * Sets the window saturation level to @p saturation.
+     * If you want to modify the existing saturation level consider using multiplySaturation.
+     * @param saturation The new saturation level
+     * @since 4.10
+     **/
+    void setSaturation(qreal saturation) const;
+    /**
+     * Multiplies the current saturation with @p factor.
+     * @param factor with which the saturation should be multiplied
+     * @return New saturation level
+     * @since 4.10
+     **/
+    qreal multiplySaturation(qreal factor);
     /**
      * Brightness of the window, in range [0; 1]
      * 1 means that the window is unchanged, 0 means that it's completely
      * black. 0.5 would make it 50% darker than usual
      **/
-    double brightness;
+    qreal brightness() const;
+    /**
+     * Sets the window brightness level to @p brightness.
+     * If you want to modify the existing brightness level consider using multiplyBrightness.
+     * @param brightness The new brightness level
+     **/
+    void setBrightness(qreal brightness);
+    /**
+     * Multiplies the current brightness level with @p factor.
+     * @param factor with which the brightness should be multiplied.
+     * @return New brightness level
+     * @since 4.10
+     **/
+    qreal multiplyBrightness(qreal factor);
+    /**
+     * The screen number for which the painting should be done.
+     * This affects color correction (different screens may need different
+     * color correction lookup tables because they have different ICC profiles).
+     * @return screen for which painting should be done
+     */
+    int screen() const;
+    /**
+     * @param screen New screen number
+     * A value less than 0 will indicate that a default profile should be done.
+     */
+    void setScreen(int screen) const;
     WindowQuadList quads;
     /**
      * Shader to be used for rendering, if any.
      */
     GLShader* shader;
-    RotationData* rotation;
+private:
+    WindowPaintDataPrivate * const d;
 };
 
-class KWIN_EXPORT ScreenPaintData
+class KWIN_EXPORT ScreenPaintData : public PaintData
 {
 public:
     ScreenPaintData();
-    double xScale;
-    double yScale;
-    double zScale;
-    int xTranslate;
-    int yTranslate;
-    double zTranslate;
-    RotationData* rotation;
+    ScreenPaintData(const ScreenPaintData &other);
+    /**
+     * Scales the screen by @p scale factor.
+     * Multiplies all three components by the given factor.
+     * @since 4.10
+     **/
+    ScreenPaintData& operator*=(qreal scale);
+    /**
+     * Scales the screen by @p scale factor.
+     * Performs a component wise multiplication on x and y components.
+     * @since 4.10
+     **/
+    ScreenPaintData& operator*=(const QVector2D &scale);
+    /**
+     * Scales the screen by @p scale factor.
+     * Performs a component wise multiplication.
+     * @since 4.10
+     **/
+    ScreenPaintData& operator*=(const QVector3D &scale);
+    /**
+     * Translates the screen by the given @p translation and returns a reference to the ScreenPaintData.
+     * @since 4.10
+     **/
+    ScreenPaintData& operator+=(const QPointF &translation);
+    /**
+     * Translates the screen by the given @p translation and returns a reference to the ScreenPaintData.
+     * Overloaded method for convenience.
+     * @since 4.10
+     **/
+    ScreenPaintData& operator+=(const QPoint &translation);
+    /**
+     * Translates the screen by the given @p translation and returns a reference to the ScreenPaintData.
+     * Overloaded method for convenience.
+     * @since 4.10
+     **/
+    ScreenPaintData& operator+=(const QVector2D &translation);
+    /**
+     * Translates the screen by the given @p translation and returns a reference to the ScreenPaintData.
+     * Overloaded method for convenience.
+     * @since 4.10
+     **/
+    ScreenPaintData& operator+=(const QVector3D &translation);
+    ScreenPaintData& operator=(const ScreenPaintData &rhs);
 };
 
 class KWIN_EXPORT ScreenPrePaintData
@@ -1709,22 +2026,6 @@ public:
     QRegion paint;
 };
 
-class KWIN_EXPORT RotationData
-{
-public:
-    RotationData();
-    enum RotationAxis {
-        XAxis,
-        YAxis,
-        ZAxis
-    };
-    RotationAxis axis;
-    float angle;
-    float xRotationPoint;
-    float yRotationPoint;
-    float zRotationPoint;
-};
-
 /**
  * @short Helper class for restricting painting area only to allowed area.
  *
