@@ -24,19 +24,35 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 // Resolves given function, using getProcAddress
-#define GL_RESOLVE( function )  function = (function ## _func)getProcAddress( #function );
-// Same as above but tries to use function "backup" if "function" doesn't exist
-// Useful when same functionality is also defined in an extension
-#define GL_RESOLVE_WITH_EXT( function, backup ) \
-    function = (function ## _func)getProcAddress( #function ); \
-    if ( !function ) \
-        function = (function ## _func)getProcAddress( #backup );
+#ifdef KWIN_HAVE_EGL
+#define GL_RESOLVE( function ) \
+    if (platformInterface == GlxPlatformInterface) \
+        function = (function ## _func)getProcAddress( #function ); \
+    else if (platformInterface == EglPlatformInterface) \
+        function = (function ## _func)eglGetProcAddress( #function );
+// Same as above but tries to use function "symbolName"
+// Useful when functionality is defined in an extension with a different name
+#define GL_RESOLVE_WITH_EXT( function, symbolName ) \
+    if (platformInterface == GlxPlatformInterface) { \
+        function = (function ## _func)getProcAddress( #symbolName ); \
+    } else if (platformInterface == EglPlatformInterface) { \
+        function = (function ## _func)eglGetProcAddress( #symbolName ); \
+    }
+#else
+// same without the switch to egl
+#define GL_RESOLVE( function ) function = (function ## _func)getProcAddress( #function );
+
+#define GL_RESOLVE_WITH_EXT( function, symbolName ) \
+    function = (function ## _func)getProcAddress( #symbolName );
+#endif
+
+#define EGL_RESOLVE( function )  function = (function ## _func)eglGetProcAddress( #function );
 
-#ifndef KWIN_HAVE_OPENGLES
 
 namespace KWin
 {
 
+#ifndef KWIN_HAVE_OPENGLES
 // Function pointers
 glXGetProcAddress_func glXGetProcAddress;
 // GLX 1.3
@@ -50,18 +66,9 @@ glXCopySubBuffer_func glXCopySubBuffer;
 glXGetVideoSync_func glXGetVideoSync;
 glXWaitVideoSync_func glXWaitVideoSync;
 glXSwapInterval_func glXSwapInterval;
-// GLX_SGIX_fbconfig
-glXGetFBConfigAttrib_func glXGetFBConfigAttrib;
-glXGetVisualFromFBConfig_func glXGetVisualFromFBConfig;
-glXCreateWindow_func glXCreateWindow;
-glXDestroyWindow_func glXDestroyWindow;
-glXCreatePixmap_func glXCreatePixmap;
-glXDestroyPixmap_func glXDestroyPixmap;
-glXGetFBConfigs_func glXGetFBConfigs;
 
 // glActiveTexture
 glActiveTexture_func glActiveTexture;
-glBlendColor_func glBlendColor;
 // framebuffer_object extension functions
 glIsRenderbuffer_func glIsRenderbuffer;
 glBindRenderbuffer_func glBindRenderbuffer;
@@ -174,141 +181,12 @@ void glxResolveFunctions()
     } else {
         glXSwapInterval = NULL;
     }
-
-    GL_RESOLVE_WITH_EXT(glXGetFBConfigAttrib, glXGetFBConfigAttribSGIX);
-    GL_RESOLVE_WITH_EXT(glXGetVisualFromFBConfig, glXGetVisualFromFBConfigSGIX);
-    GL_RESOLVE(glXGetFBConfigs);
-    GL_RESOLVE(glXCreateWindow);
-    GL_RESOLVE(glXDestroyWindow);
-    GL_RESOLVE(glXCreatePixmap);
-    GL_RESOLVE(glXDestroyPixmap);
 }
+#endif
 
-void glResolveFunctions()
-{
-    if (hasGLExtension("GL_ARB_multitexture")) {
-        GL_RESOLVE_WITH_EXT(glActiveTexture, glActiveTextureARB);
-        // Get number of texture units
-        glGetIntegerv(GL_MAX_TEXTURE_UNITS, &glTextureUnitsCount);
-    } else {
-        glActiveTexture = NULL;
-        glTextureUnitsCount = 0;
-    }
-    if (hasGLExtension("GL_EXT_framebuffer_object")) {
-        glIsRenderbuffer = (glIsRenderbuffer_func) getProcAddress("glIsRenderbufferEXT");
-        glBindRenderbuffer = (glBindRenderbuffer_func) getProcAddress("glBindRenderbufferEXT");
-        glDeleteRenderbuffers = (glDeleteRenderbuffers_func) getProcAddress("glDeleteRenderbuffersEXT");
-        glGenRenderbuffers = (glGenRenderbuffers_func) getProcAddress("glGenRenderbuffersEXT");
 
-        glRenderbufferStorage = (glRenderbufferStorage_func) getProcAddress("glRenderbufferStorageEXT");
 
-        glGetRenderbufferParameteriv = (glGetRenderbufferParameteriv_func) getProcAddress("glGetRenderbufferParameterivEXT");
-
-        glIsFramebuffer = (glIsFramebuffer_func) getProcAddress("glIsFramebufferEXT");
-        glBindFramebuffer = (glBindFramebuffer_func) getProcAddress("glBindFramebufferEXT");
-        glDeleteFramebuffers = (glDeleteFramebuffers_func) getProcAddress("glDeleteFramebuffersEXT");
-        glGenFramebuffers = (glGenFramebuffers_func) getProcAddress("glGenFramebuffersEXT");
-
-        glCheckFramebufferStatus = (glCheckFramebufferStatus_func) getProcAddress("glCheckFramebufferStatusEXT");
-
-        glFramebufferTexture1D = (glFramebufferTexture1D_func) getProcAddress("glFramebufferTexture1DEXT");
-        glFramebufferTexture2D = (glFramebufferTexture2D_func) getProcAddress("glFramebufferTexture2DEXT");
-        glFramebufferTexture3D = (glFramebufferTexture3D_func) getProcAddress("glFramebufferTexture3DEXT");
-
-        glFramebufferRenderbuffer = (glFramebufferRenderbuffer_func) getProcAddress("glFramebufferRenderbufferEXT");
-
-        glGetFramebufferAttachmentParameteriv = (glGetFramebufferAttachmentParameteriv_func) getProcAddress("glGetFramebufferAttachmentParameterivEXT");
-
-        glGenerateMipmap = (glGenerateMipmap_func) getProcAddress("glGenerateMipmapEXT");
-    } else {
-        glIsRenderbuffer = NULL;
-        glBindRenderbuffer = NULL;
-        glDeleteRenderbuffers = NULL;
-        glGenRenderbuffers = NULL;
-        glRenderbufferStorage = NULL;
-        glGetRenderbufferParameteriv = NULL;
-        glIsFramebuffer = NULL;
-        glBindFramebuffer = NULL;
-        glDeleteFramebuffers = NULL;
-        glGenFramebuffers = NULL;
-        glCheckFramebufferStatus = NULL;
-        glFramebufferTexture1D = NULL;
-        glFramebufferTexture2D = NULL;
-        glFramebufferTexture3D = NULL;
-        glFramebufferRenderbuffer = NULL;
-        glGetFramebufferAttachmentParameteriv = NULL;
-        glGenerateMipmap = NULL;
-    }
-
-    if (hasGLExtension("GL_ARB_framebuffer_object")) {
-        glBlitFramebuffer = (glBlitFramebuffer_func) getProcAddress("glBlitFramebuffer");
-    } else if (hasGLExtension("GL_EXT_framebuffer_blit")) {
-        glBlitFramebuffer = (glBlitFramebuffer_func) getProcAddress("glBlitFramebufferEXT");
-    } else {
-        glBlitFramebuffer = NULL;
-    }
-
-    if (hasGLExtension("GL_ARB_shading_language_100") && hasGLExtension("GL_ARB_fragment_shader")) {
-        GL_RESOLVE_WITH_EXT(glCreateShader, glCreateShaderObjectARB);
-        GL_RESOLVE_WITH_EXT(glShaderSource, glShaderSourceARB);
-        GL_RESOLVE_WITH_EXT(glCompileShader, glCompileShaderARB);
-        GL_RESOLVE_WITH_EXT(glDeleteShader, glDeleteObjectARB);
-        GL_RESOLVE_WITH_EXT(glCreateProgram, glCreateProgramObjectARB);
-        GL_RESOLVE_WITH_EXT(glAttachShader, glAttachObjectARB);
-        GL_RESOLVE_WITH_EXT(glLinkProgram, glLinkProgramARB);
-        GL_RESOLVE_WITH_EXT(glUseProgram, glUseProgramObjectARB);
-        GL_RESOLVE_WITH_EXT(glDeleteProgram, glDeleteObjectARB);
-        GL_RESOLVE_WITH_EXT(glGetShaderInfoLog, glGetInfoLogARB);
-        GL_RESOLVE_WITH_EXT(glGetProgramInfoLog, glGetInfoLogARB);
-        GL_RESOLVE_WITH_EXT(glGetProgramiv, glGetObjectParameterivARB);
-        GL_RESOLVE_WITH_EXT(glGetShaderiv, glGetObjectParameterivARB);
-        GL_RESOLVE_WITH_EXT(glUniform1f, glUniform1fARB);
-        GL_RESOLVE_WITH_EXT(glUniform2f, glUniform2fARB);
-        GL_RESOLVE_WITH_EXT(glUniform3f, glUniform3fARB);
-        GL_RESOLVE_WITH_EXT(glUniform4f, glUniform4fARB);
-        GL_RESOLVE_WITH_EXT(glUniform1i, glUniform1iARB);
-        GL_RESOLVE_WITH_EXT(glUniform1fv, glUniform1fvARB);
-        GL_RESOLVE_WITH_EXT(glUniform2fv, glUniform2fvARB);
-        GL_RESOLVE_WITH_EXT(glUniform3fv, glUniform3fvARB);
-        GL_RESOLVE_WITH_EXT(glUniform4fv, glUniform4fvARB);
-        GL_RESOLVE_WITH_EXT(glUniformMatrix4fv, glUniformMatrix4fvARB);
-        GL_RESOLVE_WITH_EXT(glValidateProgram, glValidateProgramARB);
-        GL_RESOLVE_WITH_EXT(glGetUniformLocation, glGetUniformLocationARB);
-        GL_RESOLVE_WITH_EXT(glVertexAttrib1f, glVertexAttrib1fARB);
-        GL_RESOLVE_WITH_EXT(glGetAttribLocation, glGetAttribLocationARB);
-        GL_RESOLVE_WITH_EXT(glBindAttribLocation, glBindAttribLocationARB);
-        GL_RESOLVE_WITH_EXT(glGetUniformfv, glGetUniformfvARB);
-    }
-    if (hasGLExtension("GL_ARB_fragment_program") && hasGLExtension("GL_ARB_vertex_program")) {
-        GL_RESOLVE(glProgramStringARB);
-        GL_RESOLVE(glBindProgramARB);
-        GL_RESOLVE(glDeleteProgramsARB);
-        GL_RESOLVE(glGenProgramsARB);
-        GL_RESOLVE(glProgramLocalParameter4fARB);
-        GL_RESOLVE(glGetProgramivARB);
-    }
-    GL_RESOLVE_WITH_EXT(glBlendColor, glBlendColorARB);
-    if (hasGLExtension("GL_ARB_vertex_buffer_object")) {
-        GL_RESOLVE_WITH_EXT(glGenBuffers, glGenBuffersARB);
-        GL_RESOLVE_WITH_EXT(glDeleteBuffers, glDeleteBuffersARB);
-        GL_RESOLVE_WITH_EXT(glBindBuffer, glBindBufferARB);
-        GL_RESOLVE_WITH_EXT(glBufferData, glBufferDataARB);
-        GL_RESOLVE_WITH_EXT(glEnableVertexAttribArray, glEnableVertexAttribArrayARB);
-        GL_RESOLVE_WITH_EXT(glDisableVertexAttribArray, glDisableVertexAttribArrayARB);
-        GL_RESOLVE_WITH_EXT(glVertexAttribPointer, glVertexAttribPointerARB);
-    } else {
-        glGenBuffers = NULL;
-        glDeleteBuffers = NULL;
-        glBindBuffer = NULL;
-        glBufferData = NULL;
-    }
-}
-
-} // namespace
-
-#else
-namespace KWin
-{
+#ifdef KWIN_HAVE_EGL
 
 // EGL
 eglCreateImageKHR_func eglCreateImageKHR;
@@ -335,15 +213,215 @@ void eglResolveFunctions()
         eglPostSubBufferNV = NULL;
     }
 }
+#endif
 
-void glResolveFunctions()
+void glResolveFunctions(OpenGLPlatformInterface platformInterface)
 {
-    if (hasGLExtension("GL_OES_EGL_image")) {
-        glEGLImageTargetTexture2DOES = (glEGLImageTargetTexture2DOES_func)eglGetProcAddress("glEGLImageTargetTexture2DOES");
+#ifndef KWIN_HAVE_OPENGLES
+    if (hasGLExtension("GL_ARB_multitexture")) {
+        GL_RESOLVE_WITH_EXT(glActiveTexture, glActiveTextureARB);
+        // Get number of texture units
+        glGetIntegerv(GL_MAX_TEXTURE_UNITS, &glTextureUnitsCount);
     } else {
-        glEGLImageTargetTexture2DOES = NULL;
+        glActiveTexture = NULL;
+        glTextureUnitsCount = 0;
     }
+
+    if (hasGLExtension("GL_ARB_framebuffer_object")) {
+        // see http://www.opengl.org/registry/specs/ARB/framebuffer_object.txt
+        GL_RESOLVE(glIsRenderbuffer);
+        GL_RESOLVE(glBindRenderbuffer);
+        GL_RESOLVE(glDeleteRenderbuffers);
+        GL_RESOLVE(glGenRenderbuffers);
+
+        GL_RESOLVE(glRenderbufferStorage);
+
+        GL_RESOLVE(glGetRenderbufferParameteriv);
+
+        GL_RESOLVE(glIsFramebuffer);
+        GL_RESOLVE(glBindFramebuffer);
+        GL_RESOLVE(glDeleteFramebuffers);
+        GL_RESOLVE(glGenFramebuffers);
+
+        GL_RESOLVE(glCheckFramebufferStatus);
+
+        GL_RESOLVE(glFramebufferTexture1D);
+        GL_RESOLVE(glFramebufferTexture2D);
+        GL_RESOLVE(glFramebufferTexture3D);
+
+        GL_RESOLVE(glFramebufferRenderbuffer);
+
+        GL_RESOLVE(glGetFramebufferAttachmentParameteriv);
+
+        GL_RESOLVE(glGenerateMipmap);
+    } else if (hasGLExtension("GL_EXT_framebuffer_object")) {
+        // see http://www.opengl.org/registry/specs/EXT/framebuffer_object.txt
+        GL_RESOLVE_WITH_EXT(glIsRenderbuffer, glIsRenderbufferEXT);
+        GL_RESOLVE_WITH_EXT(glBindRenderbuffer, glBindRenderbufferEXT);
+        GL_RESOLVE_WITH_EXT(glDeleteRenderbuffers, glDeleteRenderbuffersEXT);
+        GL_RESOLVE_WITH_EXT(glGenRenderbuffers, glGenRenderbuffersEXT);
+
+        GL_RESOLVE_WITH_EXT(glRenderbufferStorage, glRenderbufferStorageEXT);
+
+        GL_RESOLVE_WITH_EXT(glGetRenderbufferParameteriv, glGetRenderbufferParameterivEXT);
+
+        GL_RESOLVE_WITH_EXT(glIsFramebuffer, glIsFramebufferEXT);
+        GL_RESOLVE_WITH_EXT(glBindFramebuffer, glBindFramebufferEXT);
+        GL_RESOLVE_WITH_EXT(glDeleteFramebuffers, glDeleteFramebuffersEXT);
+        GL_RESOLVE_WITH_EXT(glGenFramebuffers, glGenFramebuffersEXT);
+
+        GL_RESOLVE_WITH_EXT(glCheckFramebufferStatus, glCheckFramebufferStatusEXT);
+
+        GL_RESOLVE_WITH_EXT(glFramebufferTexture1D, glFramebufferTexture1DEXT);
+        GL_RESOLVE_WITH_EXT(glFramebufferTexture2D, glFramebufferTexture2DEXT);
+        GL_RESOLVE_WITH_EXT(glFramebufferTexture3D, glFramebufferTexture3DEXT);
+
+        GL_RESOLVE_WITH_EXT(glFramebufferRenderbuffer, glFramebufferRenderbufferEXT);
+
+        GL_RESOLVE_WITH_EXT(glGetFramebufferAttachmentParameteriv, glGetFramebufferAttachmentParameterivEXT);
+
+        GL_RESOLVE_WITH_EXT(glGenerateMipmap, glGenerateMipmapEXT);
+    } else {
+        glIsRenderbuffer = NULL;
+        glBindRenderbuffer = NULL;
+        glDeleteRenderbuffers = NULL;
+        glGenRenderbuffers = NULL;
+        glRenderbufferStorage = NULL;
+        glGetRenderbufferParameteriv = NULL;
+        glIsFramebuffer = NULL;
+        glBindFramebuffer = NULL;
+        glDeleteFramebuffers = NULL;
+        glGenFramebuffers = NULL;
+        glCheckFramebufferStatus = NULL;
+        glFramebufferTexture1D = NULL;
+        glFramebufferTexture2D = NULL;
+        glFramebufferTexture3D = NULL;
+        glFramebufferRenderbuffer = NULL;
+        glGetFramebufferAttachmentParameteriv = NULL;
+        glGenerateMipmap = NULL;
+    }
+
+    if (hasGLExtension("GL_ARB_framebuffer_object")) {
+        // see http://www.opengl.org/registry/specs/ARB/framebuffer_object.txt
+        GL_RESOLVE(glBlitFramebuffer);
+    } else if (hasGLExtension("GL_EXT_framebuffer_blit")) {
+        // see http://www.opengl.org/registry/specs/EXT/framebuffer_blit.txt
+        GL_RESOLVE_WITH_EXT(glBlitFramebuffer, glBlitFramebufferEXT);
+    } else {
+        glBlitFramebuffer = NULL;
+    }
+
+    if (hasGLExtension("GL_ARB_shader_objects")) {
+        // see http://www.opengl.org/registry/specs/ARB/shader_objects.txt
+        GL_RESOLVE_WITH_EXT(glCreateShader, glCreateShaderObjectARB);
+        GL_RESOLVE_WITH_EXT(glShaderSource, glShaderSourceARB);
+        GL_RESOLVE_WITH_EXT(glCompileShader, glCompileShaderARB);
+        GL_RESOLVE_WITH_EXT(glDeleteShader, glDeleteObjectARB);
+        GL_RESOLVE_WITH_EXT(glCreateProgram, glCreateProgramObjectARB);
+        GL_RESOLVE_WITH_EXT(glAttachShader, glAttachObjectARB);
+        GL_RESOLVE_WITH_EXT(glLinkProgram, glLinkProgramARB);
+        GL_RESOLVE_WITH_EXT(glUseProgram, glUseProgramObjectARB);
+        GL_RESOLVE_WITH_EXT(glDeleteProgram, glDeleteObjectARB);
+        GL_RESOLVE_WITH_EXT(glGetShaderInfoLog, glGetInfoLogARB);
+        GL_RESOLVE_WITH_EXT(glGetProgramInfoLog, glGetInfoLogARB);
+        GL_RESOLVE_WITH_EXT(glGetProgramiv, glGetObjectParameterivARB);
+        GL_RESOLVE_WITH_EXT(glGetShaderiv, glGetObjectParameterivARB);
+        GL_RESOLVE_WITH_EXT(glUniform1f, glUniform1fARB);
+        GL_RESOLVE_WITH_EXT(glUniform2f, glUniform2fARB);
+        GL_RESOLVE_WITH_EXT(glUniform3f, glUniform3fARB);
+        GL_RESOLVE_WITH_EXT(glUniform4f, glUniform4fARB);
+        GL_RESOLVE_WITH_EXT(glUniform1i, glUniform1iARB);
+        GL_RESOLVE_WITH_EXT(glUniform1fv, glUniform1fvARB);
+        GL_RESOLVE_WITH_EXT(glUniform2fv, glUniform2fvARB);
+        GL_RESOLVE_WITH_EXT(glUniform3fv, glUniform3fvARB);
+        GL_RESOLVE_WITH_EXT(glUniform4fv, glUniform4fvARB);
+        GL_RESOLVE_WITH_EXT(glUniformMatrix4fv, glUniformMatrix4fvARB);
+        GL_RESOLVE_WITH_EXT(glValidateProgram, glValidateProgramARB);
+        GL_RESOLVE_WITH_EXT(glGetUniformLocation, glGetUniformLocationARB);
+        GL_RESOLVE_WITH_EXT(glGetUniformfv, glGetUniformfvARB);
+    } else {
+        glCreateShader = NULL;
+        glShaderSource = NULL;
+        glCompileShader = NULL;
+        glDeleteShader = NULL;
+        glCreateProgram = NULL;
+        glAttachShader = NULL;
+        glLinkProgram = NULL;
+        glUseProgram = NULL;
+        glDeleteProgram = NULL;
+        glGetShaderInfoLog = NULL;
+        glGetProgramInfoLog = NULL;
+        glGetProgramiv = NULL;
+        glGetShaderiv = NULL;
+        glUniform1f = NULL;
+        glUniform2f = NULL;
+        glUniform3f = NULL;
+        glUniform4f = NULL;
+        glUniform1i = NULL;
+        glUniform1fv = NULL;
+        glUniform2fv = NULL;
+        glUniform3fv = NULL;
+        glUniform4fv = NULL;
+        glUniformMatrix4fv = NULL;
+        glValidateProgram = NULL;
+        glGetUniformLocation = NULL;
+        glGetUniformfv = NULL;
+    }
+    if (hasGLExtension("GL_ARB_vertex_shader")) {
+        // see http://www.opengl.org/registry/specs/ARB/vertex_shader.txt
+        GL_RESOLVE_WITH_EXT(glVertexAttrib1f, glVertexAttrib1fARB);
+        GL_RESOLVE_WITH_EXT(glBindAttribLocation, glBindAttribLocationARB);
+        GL_RESOLVE_WITH_EXT(glGetAttribLocation, glGetAttribLocationARB);
+        GL_RESOLVE_WITH_EXT(glEnableVertexAttribArray, glEnableVertexAttribArrayARB);
+        GL_RESOLVE_WITH_EXT(glDisableVertexAttribArray, glDisableVertexAttribArrayARB);
+        GL_RESOLVE_WITH_EXT(glVertexAttribPointer, glVertexAttribPointerARB);
+    } else {
+        glVertexAttrib1f = NULL;
+        glBindAttribLocation = NULL;
+        glGetAttribLocation = NULL;
+        glEnableVertexAttribArray = NULL;
+        glDisableVertexAttribArray = NULL;
+        glVertexAttribPointer = NULL;
+    }
+    if (hasGLExtension("GL_ARB_fragment_program") && hasGLExtension("GL_ARB_vertex_program")) {
+        // see http://www.opengl.org/registry/specs/ARB/fragment_program.txt
+        GL_RESOLVE(glProgramStringARB);
+        GL_RESOLVE(glBindProgramARB);
+        GL_RESOLVE(glDeleteProgramsARB);
+        GL_RESOLVE(glGenProgramsARB);
+        GL_RESOLVE(glProgramLocalParameter4fARB);
+        GL_RESOLVE(glGetProgramivARB);
+    } else {
+        glProgramStringARB = NULL;
+        glBindProgramARB = NULL;
+        glDeleteProgramsARB = NULL;
+        glGenProgramsARB = NULL;
+        glProgramLocalParameter4fARB = NULL;
+        glGetProgramivARB = NULL;
+    }
+    if (hasGLExtension("GL_ARB_vertex_buffer_object")) {
+        // see http://www.opengl.org/registry/specs/ARB/vertex_buffer_object.txt
+        GL_RESOLVE_WITH_EXT(glGenBuffers, glGenBuffersARB);
+        GL_RESOLVE_WITH_EXT(glDeleteBuffers, glDeleteBuffersARB);
+        GL_RESOLVE_WITH_EXT(glBindBuffer, glBindBufferARB);
+        GL_RESOLVE_WITH_EXT(glBufferData, glBufferDataARB);
+    } else {
+        glGenBuffers = NULL;
+        glDeleteBuffers = NULL;
+        glBindBuffer = NULL;
+        glBufferData = NULL;
+    }
+#endif
+
+#ifdef KWIN_HAVE_EGL
+    if (platformInterface == EglPlatformInterface) {
+        if (hasGLExtension("GL_OES_EGL_image")) {
+            glEGLImageTargetTexture2DOES = (glEGLImageTargetTexture2DOES_func)eglGetProcAddress("glEGLImageTargetTexture2DOES");
+        } else {
+            glEGLImageTargetTexture2DOES = NULL;
+        }
+    }
+#endif
 }
 
 } // namespace
-#endif
