@@ -22,9 +22,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define KWIN_GLUTILS_FUNCS_H
 
 #include <kdemacros.h>
+#include <kwinconfig.h>
+#include <kwinglobals.h>
 
 #define KWIN_EXPORT KDE_EXPORT
 
+// common functionality
+namespace KWin {
+
+void KWIN_EXPORT glResolveFunctions(OpenGLPlatformInterface platformInterface);
+
+}
+
 #ifndef KWIN_HAVE_OPENGLES
 
 #include <GL/gl.h>
@@ -73,7 +82,6 @@ namespace KWin
 {
 
 void KWIN_EXPORT glxResolveFunctions();
-void KWIN_EXPORT glResolveFunctions();
 
 
 // Defines
@@ -201,30 +209,10 @@ typedef int (*glXSwapInterval_func)(int ratio);
 extern KWIN_EXPORT glXGetVideoSync_func glXGetVideoSync;
 extern KWIN_EXPORT glXWaitVideoSync_func glXWaitVideoSync;
 extern KWIN_EXPORT glXSwapInterval_func glXSwapInterval;
-// GLX_SGIX_fbconfig and misc GLX 1.3 stuff
-typedef int (*glXGetFBConfigAttrib_func)(Display *dpy, GLXFBConfig config,
-        int attribute, int *value);
-typedef XVisualInfo*(*glXGetVisualFromFBConfig_func)(Display *dpy, GLXFBConfig config);
-extern KWIN_EXPORT glXGetFBConfigAttrib_func glXGetFBConfigAttrib;
-extern KWIN_EXPORT glXGetVisualFromFBConfig_func glXGetVisualFromFBConfig;
-typedef GLXWindow(*glXCreateWindow_func)(Display *dpy, GLXFBConfig config,
-        Window win, const int *attribList);
-typedef void (*glXDestroyWindow_func)(Display *dpy, GLXWindow window);
-typedef GLXPixmap(*glXCreatePixmap_func)(Display *dpy, GLXFBConfig config,
-        Pixmap pixmap, const int *attribList);
-typedef void (*glXDestroyPixmap_func)(Display *dpy, GLXPixmap pixmap);
-typedef GLXFBConfig*(*glXGetFBConfigs_func)(Display *dpy, int screen, int *nelements);
-extern KWIN_EXPORT glXCreateWindow_func glXCreateWindow;
-extern KWIN_EXPORT glXDestroyWindow_func glXDestroyWindow;
-extern KWIN_EXPORT glXCreatePixmap_func glXCreatePixmap;
-extern KWIN_EXPORT glXDestroyPixmap_func glXDestroyPixmap;
-extern KWIN_EXPORT glXGetFBConfigs_func glXGetFBConfigs;
 
 // glActiveTexture
 typedef void (*glActiveTexture_func)(GLenum);
 extern KWIN_EXPORT glActiveTexture_func glActiveTexture;
-typedef void (*glBlendColor_func)(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
-extern KWIN_EXPORT glBlendColor_func glBlendColor;
 // framebuffer_object extension functions
 typedef GLboolean(*glIsRenderbuffer_func)(GLuint renderbuffer);
 typedef void (*glBindRenderbuffer_func)(GLenum target, GLuint renderbuffer);
@@ -352,11 +340,17 @@ extern KWIN_EXPORT glVertexAttribPointer_func glVertexAttribPointer;
 
 } // namespace
 
-#else
+#endif // not KWIN_HAVE_OPENGLES
+
+#ifdef KWIN_HAVE_EGL
 #define EGL_EGLEXT_PROTOTYPES
 #define GL_GLEXT_PROTOTYPES
+
+#ifdef KWIN_HAVE_OPENGLES
 #include <GLES2/gl2.h>
 #include <GLES2/gl2ext.h>
+#endif
+
 #include <EGL/egl.h>
 #include <EGL/eglext.h>
 #include <fixx11h.h>
@@ -369,7 +363,6 @@ namespace KWin
 {
 
 void KWIN_EXPORT eglResolveFunctions();
-void KWIN_EXPORT glResolveFunctions();
 // EGL
 typedef EGLImageKHR(*eglCreateImageKHR_func)(EGLDisplay, EGLContext, EGLenum, EGLClientBuffer, const EGLint*);
 typedef EGLBoolean(*eglDestroyImageKHR_func)(EGLDisplay, EGLImageKHR);
@@ -384,6 +377,6 @@ extern KWIN_EXPORT glEGLImageTargetTexture2DOES_func glEGLImageTargetTexture2DOE
 
 } // namespace
 
-#endif
+#endif // KWIN_HAVE_EGL
 
-#endif
+#endif // KWIN_GLUTILS_FUNCS_H
