@@ -24,8 +24,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "tabboxconfig.h"
 #include "tabboxhandler.h"
 // Qt
-// TODO: remove with Qt 5, only for HTML escaping the caption
-#include <QTextDocument>
 #include <QTextStream>
 // KDE
 #include <KLocale>
@@ -71,13 +69,8 @@ QVariant ClientModel::data(const QModelIndex& index, int role) const
     }
     switch(role) {
     case Qt::DisplayRole:
-    case CaptionRole: {
-        QString caption = client->caption();
-        if (Qt::mightBeRichText(caption)) {
-            caption = Qt::escape(caption);
-        }
-        return caption;
-    }
+    case CaptionRole:
+        return client->caption();
     case ClientRole:
         return qVariantFromValue((void*)client.data());
     case DesktopNameRole: {
@@ -113,39 +106,13 @@ QString ClientModel::longestCaption() const
 int ClientModel::columnCount(const QModelIndex& parent) const
 {
     Q_UNUSED(parent)
-    int count = 1;
-    switch(tabBox->config().layout()) {
-    case TabBoxConfig::HorizontalLayout:
-        count = m_clientList.count();
-        break;
-    case TabBoxConfig::VerticalLayout:
-        count = 1;
-        break;
-    case TabBoxConfig::HorizontalVerticalLayout:
-        count = qRound(sqrt(float(m_clientList.count())));
-        if (count * count < m_clientList.count())
-            count++;
-        break;
-    }
-    return qMax(count, 1);
+    return 1;
 }
 
 int ClientModel::rowCount(const QModelIndex& parent) const
 {
     Q_UNUSED(parent)
-    int count = 1;
-    switch(tabBox->config().layout()) {
-    case TabBoxConfig::HorizontalLayout:
-        count = 1;
-        break;
-    case TabBoxConfig::VerticalLayout:
-        count = m_clientList.count();
-        break;
-    case TabBoxConfig::HorizontalVerticalLayout:
-        count = qRound(sqrt(float(m_clientList.count())));
-        break;
-    }
-    return count;
+    return m_clientList.count();
 }
 
 QModelIndex ClientModel::parent(const QModelIndex& child) const
