@@ -45,6 +45,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtDBus/QtDBus>
 
 #include "client.h"
+#include "composite.h"
 #ifdef KWIN_BUILD_TABBOX
 #include "tabbox.h"
 #endif
@@ -54,21 +55,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "outline.h"
 #include "group.h"
 #include "rules.h"
-#include "kwinadaptor.h"
+#include "dbusinterface.h"
 #include "unmanaged.h"
 #include "deleted.h"
 #include "effects.h"
 #include "overlaywindow.h"
+#include "useractions.h"
 #include <kwinglplatform.h>
 #include <kwinglutils.h>
 #ifdef KWIN_BUILD_SCRIPTING
 #include "scripting/scripting.h"
 #endif
-#ifdef KWIN_BUILD_TILING
-#include "tiling/tile.h"
-#include "tiling/tilinglayout.h"
-#include "tiling/tiling.h"
-#endif
 
 #include <X11/extensions/shape.h>
 #include <X11/keysym.h>
@@ -76,8 +73,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <X11/cursorfont.h>
 #include <QX11Info>
 #include <stdio.h>
-#include <kauthorized.h>
-#include <ktoolinvocation.h>
 #include <kglobalsettings.h>
 #include <kwindowsystem.h>
 #include <kwindowinfo.h>
@@ -88,6 +83,12 @@ namespace KWin
 extern int screen_number;
 static const int KWIN_MAX_NUMBER_DESKTOPS = 20;
 
+#ifdef KWIN_BUILD_KAPPMENU
+static const char *KDED_SERVICE = "org.kde.kded";
+static const char *KDED_APPMENU_PATH = "/modules/appmenu";
+static const char *KDED_INTERFACE = "org.kde.kded";
+#endif
+
 Workspace* Workspace::_self = 0;
 
 //-----------------------------------------------------------------------------
@@ -110,8 +111,8 @@ Workspace::Workspace(bool restore)
 #ifdef KWIN_BUILD_SCREENEDGES
     , m_screenEdgeOrientation(0)
 #endif
+    , m_compositor(NULL)
     // Unsorted
-    , m_nextFrameDelay(0)
     , active_popup(NULL)
     , active_popup_client(NULL)
     , temporaryRulesMessages("_KDE_NET_WM_TEMPORARY_RULES", NULL, false)
@@ -133,12 +134,7 @@ Workspace::Workspace(bool restore)
 #ifdef KWIN_BUILD_TABBOX
     , tab_box(0)
 #endif
-    , popup(0)
-    , advanced_popup(0)
-    , desk_popup(0)
-    , activity_popup(0)
-    , add_tabs_popup(0)
-    , switch_to_tab_popup(0)
+    , m_userActionsMenu(new UserActionsMenu(this))
     , keys(0)
     , client_keys(NULL)
     , disable_shortcuts_keys(NULL)
@@ -151,27 +147,24 @@ Workspace::Workspace(bool restore)
     , set_active_client_recursion(0)
     , block_stacking_updates(0)
     , forced_global_mouse_grab(false)
-    , cm_selection(NULL)
-    , compositingSuspended(false)
-    , compositingBlocked(false)
-    , xrrRefreshRate(0)
     , transSlider(NULL)
     , transButton(NULL)
-    , forceUnredirectCheck(true)
-    , m_finishingCompositing(false)
     , m_scripting(NULL)
 {
     // If KWin was already running it saved its configuration after loosing the selection -> Reread
     QFuture<void> reparseConfigFuture = QtConcurrent::run(options, &Options::reparseConfiguration);
 
-    (void) new KWinAdaptor(this);
-
+#ifdef KWIN_BUILD_KAPPMENU
     QDBusConnection dbus = QDBusConnection::sessionBus();
-    dbus.registerObject("/KWin", this);
-    dbus.connect(QString(), "/KWin", "org.kde.KWin", "reloadConfig",
-                 this, SLOT(slotReloadConfig()));
-    dbus.connect(QString(), "/KWin", "org.kde.KWin", "reinitCompositing",
-                 this, SLOT(slotReinitCompositing()));
+    dbus.connect(KDED_SERVICE, KDED_APPMENU_PATH, KDED_INTERFACE, "showRequest",
+                 this, SLOT(slotShowRequest(qulonglong)));
+    dbus.connect(KDED_SERVICE, KDED_APPMENU_PATH, KDED_INTERFACE, "menuAvailable",
+                 this, SLOT(slotMenuAvailable(qulonglong)));
+    dbus.connect(KDED_SERVICE, KDED_APPMENU_PATH, KDED_INTERFACE, "menuHidden",
+                 this, SLOT(slotMenuHidden(qulonglong)));
+    dbus.connect(KDED_SERVICE, KDED_APPMENU_PATH, KDED_INTERFACE, "clearMenus",
+                 this, SLOT(slotClearMenus()));
+#endif
 
     // Initialize desktop grid array
     desktopGrid_[0] = 0;
@@ -194,19 +187,10 @@ Workspace::Workspace(bool restore)
     connect(&temporaryRulesMessages, SIGNAL(gotMessage(QString)),
             this, SLOT(gotTemporaryRulesMessage(QString)));
     connect(&rulesUpdatedTimer, SIGNAL(timeout()), this, SLOT(writeWindowRules()));
-    connect(&unredirectTimer, SIGNAL(timeout()), this, SLOT(delayedCheckUnredirect()));
-    connect(&compositeResetTimer, SIGNAL(timeout()), this, SLOT(resetCompositing()));
-    unredirectTimer.setSingleShot(true);
-    compositeResetTimer.setSingleShot(true);
-
     updateXTime(); // Needed for proper initialization of user_time in Client ctor
 
     delayFocusTimer = 0;
 
-#ifdef KWIN_BUILD_TILING
-    m_tiling = new Tiling(this);
-#endif
-
     if (restore)
         loadSessionInfo();
 
@@ -227,13 +211,16 @@ Workspace::Workspace(bool restore)
                  ExposureMask
                 );
 
-    compositingSuspended = !options->isUseCompositing();
 #ifdef KWIN_BUILD_TABBOX
     // need to create the tabbox before compositing scene is setup
     tab_box = new TabBox::TabBox(this);
 #endif
 
-    setupCompositing();
+    m_compositor = Compositor::createCompositor(this);
+    connect(this, SIGNAL(currentDesktopChanged(int,KWin::Client*)), m_compositor, SLOT(addRepaintFull()));
+    connect(m_compositor, SIGNAL(compositingToggled(bool)), SLOT(slotCompositingToggled()));
+
+    new DBusInterface(this);
 
     // Compatibility
     long data = 1;
@@ -413,7 +400,6 @@ void Workspace::init()
 
     connect(&reconfigureTimer, SIGNAL(timeout()), this, SLOT(slotReconfigure()));
     connect(&updateToolWindowsTimer, SIGNAL(timeout()), this, SLOT(slotUpdateToolWindows()));
-    connect(&mousePollingTimer, SIGNAL(timeout()), SLOT(performMousePoll()));
 
     connect(KGlobalSettings::self(), SIGNAL(appearanceChanged()), this, SLOT(reconfigure()));
     connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)), this, SLOT(slotSettingsChanged(int)));
@@ -429,26 +415,50 @@ void Workspace::init()
         // Begin updates blocker block
         StackingUpdatesBlocker blocker(this);
 
-        unsigned int i, nwins;
-        Window root_return, parent_return;
-        Window* wins;
-        XQueryTree(display(), rootWindow(), &root_return, &parent_return, &wins, &nwins);
         bool fixoffset = KCmdLineArgs::parsedArgs()->getOption("crashes").toInt() > 0;
-        for (i = 0; i < nwins; i++) {
-            XWindowAttributes attr;
-            XGetWindowAttributes(display(), wins[i], &attr);
-            if (attr.override_redirect) {
-                createUnmanaged(wins[i]);
+        xcb_connection_t *conn = connection();
+
+        xcb_query_tree_reply_t *tree = xcb_query_tree_reply(conn,
+                            xcb_query_tree_unchecked(conn, rootWindow()), 0);
+        xcb_window_t *wins = xcb_query_tree_children(tree);
+
+        QVector<xcb_get_window_attributes_cookie_t> attr_cookies;
+        QVector<xcb_get_geometry_cookie_t> geom_cookies;
+
+        attr_cookies.reserve(tree->children_len);
+        geom_cookies.reserve(tree->children_len);
+
+        // Request the attributes and geometries of all toplevel windows
+        for (int i = 0; i < tree->children_len; i++) {
+            attr_cookies << xcb_get_window_attributes(conn, wins[i]);
+            geom_cookies << xcb_get_geometry(conn, wins[i]);
+        }
+
+        // Get the replies
+        for (int i = 0; i < tree->children_len; i++) {
+            ScopedCPointer<xcb_get_window_attributes_reply_t> attr
+                        = xcb_get_window_attributes_reply(conn, attr_cookies[i], 0);
+            ScopedCPointer<xcb_get_geometry_reply_t> geometry
+                        = xcb_get_geometry_reply(conn, geom_cookies[i], 0);
+
+            if (!attr || !geometry)
                 continue;
-            }
-            if (attr.map_state != IsUnmapped) {
+
+            if (attr->override_redirect) {
+                if (attr->map_state == XCB_MAP_STATE_VIEWABLE &&
+                    attr->_class != XCB_WINDOW_CLASS_INPUT_ONLY)
+                    // ### This will request the attributes again
+                    createUnmanaged(wins[i]);
+            } else if (attr->map_state != XCB_MAP_STATE_UNMAPPED) {
                 if (fixoffset)
-                    fixPositionAfterCrash(wins[ i ], attr);
+                    fixPositionAfterCrash(wins[i], geometry);
+
+                // ### This will request the attributes again
                 createClient(wins[i], true);
             }
         }
-        if (wins)
-            XFree((void*)(wins));
+
+        free(tree);
 
         // Propagate clients, will really happen at the end of the updates blocker block
         updateStackingOrder(true);
@@ -488,11 +498,6 @@ void Workspace::init()
     if (new_active_client != NULL)
         activateClient(new_active_client);
 
-#ifdef KWIN_BUILD_TILING
-    // Enable/disable tiling
-    m_tiling->setEnabled(options->isTilingOn());
-#endif
-
 
 #ifdef KWIN_BUILD_SCRIPTING
     m_scripting = new Scripting(this);
@@ -511,11 +516,9 @@ void Workspace::init()
 
 Workspace::~Workspace()
 {
-    finishCompositing();
+    delete m_compositor;
+    m_compositor = NULL;
     blockStackingUpdates(true);
-#ifdef KWIN_BUILD_TILING
-    delete m_tiling;
-#endif
 
     // TODO: grabXServer();
 
@@ -536,7 +539,6 @@ Workspace::~Workspace()
     for (UnmanagedList::iterator it = unmanaged.begin(), end = unmanaged.end(); it != end; ++it)
         (*it)->release(true);
     delete m_outline;
-    discardPopup();
     XDeleteProperty(display(), rootWindow(), atoms->kwin_running);
 
     writeWindowRules();
@@ -567,26 +569,30 @@ Client* Workspace::createClient(Window w, bool is_mapped)
 {
     StackingUpdatesBlocker blocker(this);
     Client* c = new Client(this);
+    connect(c, SIGNAL(needsRepaint()), m_compositor, SLOT(scheduleRepaint()));
+    connect(c, SIGNAL(activeChanged()), m_compositor, SLOT(checkUnredirect()));
+    connect(c, SIGNAL(fullScreenChanged()), m_compositor, SLOT(checkUnredirect()));
+    connect(c, SIGNAL(geometryChanged()), m_compositor, SLOT(checkUnredirect()));
+    connect(c, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), m_compositor, SLOT(checkUnredirect()));
+    connect(c, SIGNAL(blockingCompositingChanged(KWin::Client*)), m_compositor, SLOT(updateCompositeBlocking(KWin::Client*)));
     if (!c->manage(w, is_mapped)) {
         Client::deleteClient(c, Allowed);
         return NULL;
     }
     addClient(c, Allowed);
-#ifdef KWIN_BUILD_TILING
-    m_tiling->createTile(c);
-#endif
     return c;
 }
 
 Unmanaged* Workspace::createUnmanaged(Window w)
 {
-    if (compositing() && w == scene->overlayWindow()->window())
+    if (m_compositor && m_compositor->checkForOverlayWindow(w))
         return NULL;
     Unmanaged* c = new Unmanaged(this);
     if (!c->track(w)) {
         Unmanaged::deleteUnmanaged(c, Allowed);
         return NULL;
     }
+    connect(c, SIGNAL(needsRepaint()), m_compositor, SLOT(scheduleRepaint()));
     addUnmanaged(c, Allowed);
     emit unmanagedAdded(c);
     return c;
@@ -634,6 +640,10 @@ void Workspace::addClient(Client* c, allowed_t)
     if (tabBox()->isDisplayed())
         tab_box->reset(true);
 #endif
+#ifdef KWIN_BUILD_KAPPMENU
+        if (m_windowsMenu.removeOne(c->window()))
+            c->setAppMenuAvailable();
+#endif
 }
 
 void Workspace::addUnmanaged(Unmanaged* c, allowed_t)
@@ -651,6 +661,9 @@ void Workspace::removeClient(Client* c, allowed_t)
 
     if (c == active_popup_client)
         closeActivePopup();
+    if (m_userActionsMenu->isMenuClient(c)) {
+        m_userActionsMenu->close();
+    }
 
     c->untab();
 
@@ -698,7 +711,9 @@ void Workspace::removeClient(Client* c, allowed_t)
 
     updateStackingOrder(true);
 
-    updateCompositeBlocking();
+    if (m_compositor) {
+        m_compositor->updateCompositeBlocking();
+    }
 
 #ifdef KWIN_BUILD_TABBOX
     if (tabBox()->isDisplayed())
@@ -732,13 +747,12 @@ void Workspace::addDeleted(Deleted* c, Toplevel *orig, allowed_t)
         stacking_order.append(c);
     }
     x_stacking_dirty = true;
+    connect(c, SIGNAL(needsRepaint()), m_compositor, SLOT(scheduleRepaint()));
 }
 
 void Workspace::removeDeleted(Deleted* c, allowed_t)
 {
     assert(deleted.contains(c));
-    if (scene)
-        scene->windowDeleted(c);
     emit deletedRemoved(c);
     deleted.removeAll(c);
     unconstrained_stacking_order.removeAll(c);
@@ -926,7 +940,34 @@ void Workspace::slotReloadConfig()
 {
     reconfigure();
 }
+#ifdef KWIN_BUILD_KAPPMENU
+void Workspace::slotShowRequest(qulonglong wid)
+{
+    if (Client *c = findClient(WindowMatchPredicate(wid)))
+        c->emitShowRequest();
+}
 
+void Workspace::slotMenuAvailable(qulonglong wid)
+{
+    if (Client *c = findClient(WindowMatchPredicate(wid)))
+        c->setAppMenuAvailable();
+    else
+        m_windowsMenu.append(wid);
+}
+
+void Workspace::slotMenuHidden(qulonglong wid)
+{
+    if (Client *c = findClient(WindowMatchPredicate(wid)))
+        c->emitMenuHidden();
+}
+
+void Workspace::slotClearMenus()
+{
+    foreach (Client *c, clients) {
+       c->setAppMenuUnavailable();
+    }
+}
+#endif
 void Workspace::reconfigure()
 {
     reconfigureTimer.start(200);
@@ -944,14 +985,17 @@ bool Workspace::waitForCompositingSetup()
         reconfigureTimer.stop();
         slotReconfigure();
     }
-    return compositingActive();
+    if (m_compositor) {
+        return m_compositor->isActive();
+    }
+    return false;
 }
 
 void Workspace::slotSettingsChanged(int category)
 {
     kDebug(1212) << "Workspace::slotSettingsChanged()";
     if (category == KGlobalSettings::SETTINGS_SHORTCUTS)
-        discardPopup();
+        m_userActionsMenu->discard();
 }
 
 /**
@@ -976,7 +1020,7 @@ void Workspace::slotReconfigure()
     unsigned long changed = options->updateSettings();
 
     emit configChanged();
-    discardPopup();
+    m_userActionsMenu->discard();
     updateToolWindows(true);
 
     if (hasDecorationPlugin() && mgr->reset(changed)) {
@@ -1015,15 +1059,6 @@ void Workspace::slotReconfigure()
     }
     m_screenEdge.update();
 #endif
-
-    if (!compositingSuspended) {
-        setupCompositing();
-        if (effects)   // setupCompositing() may fail
-            effects->reconfigure();
-        addRepaintFull();
-    } else
-        finishCompositing();
-
     loadWindowRules();
     for (ClientList::Iterator it = clients.begin();
             it != clients.end();
@@ -1045,11 +1080,6 @@ void Workspace::slotReconfigure()
         }
     }
 
-#ifdef KWIN_BUILD_TILING
-    m_tiling->setEnabled(options->isTilingOn());
-    // just so that we reset windows in the right manner, 'activate' the current active window
-    m_tiling->notifyTilingWindowActivated(activeClient());
-#endif
     if (hasDecorationPlugin()) {
         rootInfo->setSupported(NET::WM2FrameOverlap, mgr->factory()->supports(AbilityExtendIntoClientArea));
     } else {
@@ -1057,48 +1087,6 @@ void Workspace::slotReconfigure()
     }
 }
 
-void Workspace::restartKWin(const QString &reason)
-{
-    kDebug(1212) << "restarting kwin for:" << reason;
-    char cmd[1024]; // copied from crashhandler - maybe not the best way to do?
-    sprintf(cmd, "%s --replace &", QFile::encodeName(QCoreApplication::applicationFilePath()).constData());
-    system(cmd);
-}
-
-void Workspace::slotReinitCompositing()
-{
-    // Reparse config. Config options will be reloaded by setupCompositing()
-    KGlobal::config()->reparseConfiguration();
-    const QString graphicsSystem = KConfigGroup(KGlobal::config(), "Compositing").readEntry("GraphicsSystem", "");
-    if ((Extensions::nonNativePixmaps() && graphicsSystem == "native") ||
-        (!Extensions::nonNativePixmaps() && (graphicsSystem == "raster" || graphicsSystem == "opengl")) ) {
-        restartKWin("explicitly reconfigured graphicsSystem change");
-        return;
-    }
-
-    // Update any settings that can be set in the compositing kcm.
-#ifdef KWIN_BUILD_SCREENEDGES
-    m_screenEdge.update();
-#endif
-
-    // Restart compositing
-    finishCompositing();
-
-    // resume compositing if suspended
-    compositingSuspended = false;
-    options->setCompositingInitialized(false);
-    setupCompositing();
-    if (hasDecorationPlugin()) {
-        KDecorationFactory* factory = mgr->factory();
-        factory->reset(SettingCompositing);
-    }
-
-    if (effects) { // setupCompositing() may fail
-        effects->reconfigure();
-        emit compositingToggled(true);
-    }
-}
-
 static bool _loading_desktop_settings = false;
 void Workspace::loadDesktopSettings()
 {
@@ -1164,35 +1152,6 @@ void Workspace::saveDesktopSettings()
     group.sync();
 }
 
-QStringList Workspace::configModules(bool controlCenter)
-{
-    QStringList args;
-    args <<  "kwindecoration";
-    if (controlCenter)
-        args << "kwinoptions";
-    else if (KAuthorized::authorizeControlModule("kde-kwinoptions.desktop"))
-        args << "kwinactions" << "kwinfocus" <<  "kwinmoving" << "kwinadvanced"
-             << "kwinrules" << "kwincompositing"
-#ifdef KWIN_BUILD_TABBOX
-             << "kwintabbox"
-#endif
-#ifdef KWIN_BUILD_SCREENEDGES
-             << "kwinscreenedges"
-#endif
-#ifdef KWIN_BUILD_SCRIPTING
-             << "kwinscripts"
-#endif
-             ;
-    return args;
-}
-
-void Workspace::configureWM()
-{
-    QStringList args;
-    args << "--icon" << "preferences-system-windows" << configModules(false);
-    KToolInvocation::kdeinitExec("kcmshell4", args);
-}
-
 /**
  * Avoids managing a window with title \a title
  */
@@ -1239,8 +1198,6 @@ unsigned int ObscuringWindows::max_cache_size = 0;
 
 void ObscuringWindows::create(Client* c)
 {
-    if (compositing())
-        return; // Not needed with compositing
     if (cached == 0)
         cached = new QList<Window>;
     Window obs_win;
@@ -1318,7 +1275,7 @@ bool Workspace::setCurrentDesktop(int new_desktop)
                 continue;
             }
             if (!c->isOnDesktop(new_desktop) && c != movingClient && c->isOnCurrentActivity()) {
-                if (c->isShown(true) && c->isOnDesktop(old_desktop))
+                if (c->isShown(true) && c->isOnDesktop(old_desktop) && !compositing())
                     obs_wins.create(c);
                 (c)->updateVisibility();
             }
@@ -1327,19 +1284,8 @@ bool Workspace::setCurrentDesktop(int new_desktop)
         // Now propagate the change, after hiding, before showing
         rootInfo->setCurrentDesktop(currentDesktop());
 
-        // if the client is moved to another desktop, that desktop may
-        // not have an existing layout. In addition this tiling layout
-        // will require rearrangement, so notify about desktop changes.
         if (movingClient && !movingClient->isOnDesktop(new_desktop)) {
-            int old_desktop = movingClient->desktop();
             movingClient->setDesktop(new_desktop);
-#ifdef KWIN_BUILD_TILING
-            if (m_tiling->isEnabled()) {
-                m_tiling->notifyTilingWindowDesktopChanged(movingClient, old_desktop);
-            }
-#else
-    Q_UNUSED(old_desktop)
-#endif
         }
 
         for (int i = stacking_order.size() - 1; i >= 0 ; --i) {
@@ -1431,9 +1377,6 @@ bool Workspace::setCurrentDesktop(int new_desktop)
     //    s += QString::number( desktop_focus_chain[i] ) + ", ";
     //kDebug( 1212 ) << s << "}\n";
 
-    if (compositing())
-        addRepaintFull();
-
     emit currentDesktopChanged(old_desktop, movingClient);
     return true;
 }
@@ -1464,19 +1407,23 @@ fetchActivityListAndCurrent(KActivities::Controller *controller)
     return CurrentAndList(c, l);
 }
 
-void Workspace::updateActivityList(bool running, bool updateCurrent, QString slot)
+void Workspace::updateActivityList(bool running, bool updateCurrent, QObject *target, QString slot)
 {
     if (updateCurrent) {
         QFutureWatcher<CurrentAndList>* watcher = new QFutureWatcher<CurrentAndList>;
         connect( watcher, SIGNAL(finished()), SLOT(handleActivityReply()) );
-        if (!slot.isEmpty())
+        if (!slot.isEmpty()) {
             watcher->setProperty("activityControllerCallback", slot); // "activity reply trigger"
+            watcher->setProperty("activityControllerCallbackTarget", qVariantFromValue((void*)target));
+        }
         watcher->setFuture(QtConcurrent::run(fetchActivityListAndCurrent, &activityController_ ));
     } else {
         QFutureWatcher<AssignedList>* watcher = new QFutureWatcher<AssignedList>;
         connect(watcher, SIGNAL(finished()), SLOT(handleActivityReply()));
-        if (!slot.isEmpty())
+        if (!slot.isEmpty()) {
             watcher->setProperty("activityControllerCallback", slot); // "activity reply trigger"
+            watcher->setProperty("activityControllerCallbackTarget", qVariantFromValue((void*)target));
+        }
         QStringList *target = running ? &openActivities_ : &allActivities_;
         watcher->setFuture(QtConcurrent::run(fetchActivityList, &activityController_, target, running));
     }
@@ -1500,9 +1447,10 @@ void Workspace::handleActivityReply()
 
     if (watcherObject) {
         QString slot = watcherObject->property("activityControllerCallback").toString();
+        QObject *target = static_cast<QObject*>(watcherObject->property("activityControllerCallbackTarget").value<void*>());
         watcherObject->deleteLater(); // has done it's job
         if (!slot.isEmpty())
-            QMetaObject::invokeMethod(this, slot.toAscii().data(), Qt::DirectConnection);
+            QMetaObject::invokeMethod(target, slot.toAscii().data(), Qt::DirectConnection);
     }
 }
 //END threaded activity list fetching
@@ -1545,7 +1493,7 @@ void Workspace::updateCurrentActivity(const QString &new_activity)
                 continue;
             }
             if (!c->isOnActivity(new_activity) && c != movingClient && c->isOnCurrentDesktop()) {
-                if (c->isShown(true) && c->isOnActivity(old_activity))
+                if (c->isShown(true) && c->isOnActivity(old_activity) && !compositing())
                     obs_wins.create(c);
                 c->updateVisibility();
             }
@@ -1557,13 +1505,7 @@ void Workspace::updateCurrentActivity(const QString &new_activity)
         /* TODO someday enable dragging windows to other activities
         if ( movingClient && !movingClient->isOnDesktop( new_desktop ))
             {
-            int old_desktop = movingClient->desktop();
             movingClient->setDesktop( new_desktop );
-            if ( tilingEnabled() )
-                {
-                notifyWindowDesktopChanged( movingClient, old_desktop );
-                }
-            }
             */
 
         for (int i = stacking_order.size() - 1; i >= 0 ; --i) {
@@ -1644,8 +1586,8 @@ void Workspace::updateCurrentActivity(const QString &new_activity)
 
     //if ( effects != NULL && old_desktop != 0 && old_desktop != new_desktop )
     //    static_cast<EffectsHandlerImpl*>( effects )->desktopChanged( old_desktop );
-    if (compositing())
-        addRepaintFull();
+    if (compositing() && m_compositor)
+        m_compositor->addRepaintFull();
 
 }
 
@@ -1780,9 +1722,6 @@ void Workspace::sendClientToDesktop(Client* c, int desk, bool dont_activate)
     } else
         raiseClient(c);
 
-#ifdef KWIN_BUILD_TILING
-    m_tiling->notifyTilingWindowDesktopChanged(c, old_desktop);
-#endif
     c->checkWorkspacePosition( QRect(), old_desktop );
 
     ClientList transients_stacking_order = ensureStackingOrder(c->transients());
@@ -1821,7 +1760,6 @@ void Workspace::toggleClientOnActivity(Client* c, const QString &activity, bool
         raiseClient(c);
 
     //notifyWindowDesktopChanged( c, old_desktop );
-    //FIXME does tiling break?
 
     ClientList transients_stacking_order = ensureStackingOrder(c->transients());
     for (ClientList::ConstIterator it = transients_stacking_order.constBegin();
@@ -1879,6 +1817,7 @@ int Workspace::screenNumber(const QPoint& pos) const
 
 void Workspace::sendClientToScreen(Client* c, int screen)
 {
+    screen = c->rules()->checkScreen(screen);
     if (c->screen() == screen)   // Don't use isOnScreen(), that's true even when only partially
         return;
     GeometryUpdatesBlocker blocker(c);
@@ -2016,49 +1955,6 @@ void Workspace::focusToNull()
     XSetInputFocus(display(), null_focus_window, RevertToPointerRoot, xTime());
 }
 
-void Workspace::helperDialog(const QString& message, const Client* c)
-{
-    QStringList args;
-    QString type;
-    if (message == "noborderaltf3") {
-        KAction* action = qobject_cast<KAction*>(keys->action("Window Operations Menu"));
-        assert(action != NULL);
-        QString shortcut = QString("%1 (%2)").arg(action->text())
-                           .arg(action->globalShortcut().primary().toString(QKeySequence::NativeText));
-        args << "--msgbox" << i18n(
-                 "You have selected to show a window without its border.\n"
-                 "Without the border, you will not be able to enable the border "
-                 "again using the mouse: use the window operations menu instead, "
-                 "activated using the %1 keyboard shortcut.",
-                 shortcut);
-        type = "altf3warning";
-    } else if (message == "fullscreenaltf3") {
-        KAction* action = qobject_cast<KAction*>(keys->action("Window Operations Menu"));
-        assert(action != NULL);
-        QString shortcut = QString("%1 (%2)").arg(action->text())
-                           .arg(action->globalShortcut().primary().toString(QKeySequence::NativeText));
-        args << "--msgbox" << i18n(
-                 "You have selected to show a window in fullscreen mode.\n"
-                 "If the application itself does not have an option to turn the fullscreen "
-                 "mode off you will not be able to disable it "
-                 "again using the mouse: use the window operations menu instead, "
-                 "activated using the %1 keyboard shortcut.",
-                 shortcut);
-        type = "altf3warning";
-    } else
-        abort();
-    if (!type.isEmpty()) {
-        KConfig cfg("kwin_dialogsrc");
-        KConfigGroup cg(&cfg, "Notification Messages");  // Depends on KMessageBox
-        if (!cg.readEntry(type, true))
-            return;
-        args << "--dontagain" << "kwin_dialogsrc:" + type;
-    }
-    if (c != NULL)
-        args << "--embed" << QString::number(c->window());
-    KProcess::startDetached("kdialog", args);
-}
-
 void Workspace::setShowingDesktop(bool showing)
 {
     rootInfo->setShowingDesktop(showing);
@@ -2262,57 +2158,6 @@ TabBox::TabBox* Workspace::tabBox() const
 }
 #endif
 
-#ifdef KWIN_BUILD_TILING
-Tiling* Workspace::tiling()
-{
-    return m_tiling;
-}
-#endif
-
-/*
- * Called from D-BUS
- */
-void Workspace::toggleTiling()
-{
-#ifdef KWIN_BUILD_TILING
-    if (m_tiling) {
-        m_tiling->slotToggleTiling();
-    }
-#endif
-}
-
-/*
- * Called from D-BUS
- */
-void Workspace::nextTileLayout()
-{
-#ifdef KWIN_BUILD_TILING
-    if (m_tiling) {
-        m_tiling->slotNextTileLayout();
-    }
-#endif
-}
-
-/*
- * Called from D-BUS
- */
-void Workspace::previousTileLayout()
-{
-#ifdef KWIN_BUILD_TILING
-    if (m_tiling) {
-        m_tiling->slotPreviousTileLayout();
-    }
-#endif
-}
-
-void Workspace::dumpTiles() const {
-#ifdef KWIN_BUILD_TILING
-    if (m_tiling) {
-        m_tiling->dumpTiles();
-    }
-#endif
-}
-
 QString Workspace::supportInformation() const
 {
     QString support;
@@ -2349,6 +2194,8 @@ QString Workspace::supportInformation() const
     if (effects) {
         support.append("Compositing is active\n");
         switch (effects->compositingType()) {
+        case OpenGL1Compositing:
+        case OpenGL2Compositing:
         case OpenGLCompositing: {
 #ifdef KWIN_HAVE_OPENGLES
             support.append("Compositing Type: OpenGL ES 2.0\n");
@@ -2414,8 +2261,14 @@ QString Workspace::supportInformation() const
             } else {
                 support.append(" no\n");
             }
+            support.append("Virtual Machine: ");
+            if (platform->isVirtualMachine()) {
+                support.append(" yes\n");
+            } else {
+                support.append(" no\n");
+            }
 
-            if (ShaderManager::instance()->isValid()) {
+            if (effects->compositingType() == OpenGL2Compositing) {
                 support.append("OpenGL 2 Shaders are used\n");
             } else {
                 support.append("OpenGL 2 Shaders are not used. Legacy OpenGL 1.x code path is used.\n");
@@ -2431,18 +2284,18 @@ QString Workspace::supportInformation() const
         }
         support.append("\nLoaded Effects:\n");
         support.append(  "---------------\n");
-        foreach (const QString &effect, loadedEffects()) {
+        foreach (const QString &effect, static_cast<EffectsHandlerImpl*>(effects)->loadedEffects()) {
             support.append(effect % '\n');
         }
         support.append("\nCurrently Active Effects:\n");
         support.append(  "-------------------------\n");
-        foreach (const QString &effect, activeEffects()) {
+        foreach (const QString &effect, static_cast<EffectsHandlerImpl*>(effects)->activeEffects()) {
             support.append(effect % '\n');
         }
         support.append("\nEffect Settings:\n");
         support.append(  "----------------\n");
-        foreach (const QString &effect, loadedEffects()) {
-            support.append(supportInformationForEffect(effect));
+        foreach (const QString &effect, static_cast<EffectsHandlerImpl*>(effects)->loadedEffects()) {
+            support.append(static_cast<EffectsHandlerImpl*>(effects)->supportInformation(effect));
             support.append('\n');
         }
     } else {
@@ -2451,6 +2304,22 @@ QString Workspace::supportInformation() const
     return support;
 }
 
+void Workspace::slotCompositingToggled()
+{
+    // notify decorations that composition state has changed
+    if (hasDecorationPlugin()) {
+        KDecorationFactory* factory = mgr->factory();
+        factory->reset(SettingCompositing);
+    }
+}
+
+void Workspace::slotToggleCompositing()
+{
+    if (m_compositor) {
+        m_compositor->slotToggleCompositing();
+    }
+}
+
 } // namespace
 
 #include "workspace.moc"
