@@ -72,18 +72,15 @@ class TabBox;
 #endif
 
 class Client;
-#ifdef KWIN_BUILD_TILING
-class Tile;
-class Tiling;
-class TilingLayout;
-#endif
 class Outline;
 class RootInfo;
 class PluginMgr;
 class Placement;
 class Rules;
 class Scripting;
+class UserActionsMenu;
 class WindowRules;
+class Compositor;
 
 class Workspace : public QObject, public KDecorationDefines
 {
@@ -196,10 +193,18 @@ public:
     const UnmanagedList &unmanagedList() const {
         return unmanaged;
     }
-
-#ifdef KWIN_BUILD_TILING
-    Tiling* tiling();
-#endif
+    /**
+     * @return List of desktop "clients" currently managed by Workspace
+     **/
+    const ClientList &desktopList() const {
+        return desktops;
+    }
+    /**
+     * @return List of deleted "clients" currently managed by Workspace
+     **/
+    const DeletedList &deletedList() const {
+        return deleted;
+    }
 
     Outline* outline();
 #ifdef KWIN_BUILD_SCREENEDGES
@@ -314,14 +319,12 @@ private:
     KActivities::Controller activityController_;
 #endif
 
-#ifdef KWIN_BUILD_TILING
-    Tiling* m_tiling;
-#endif
     Outline* m_outline;
 #ifdef KWIN_BUILD_SCREENEDGES
     ScreenEdge m_screenEdge;
     Qt::Orientations m_screenEdgeOrientation;
 #endif
+    Compositor *m_compositor;
 
     //-------------------------------------------------
     // Unsorted
@@ -339,6 +342,12 @@ public:
     QStringList activityList() const {
         return allActivities_;
     }
+    const QStringList &openActivities() const {
+        return openActivities_;
+    }
+#ifdef KWIN_BUILD_ACTIVITIES
+    void updateActivityList(bool running, bool updateCurrent, QObject *target = NULL, QString slot = QString());
+#endif
     // True when performing Workspace::updateClientArea().
     // The calls below are valid only in that case.
     bool inUpdateClientArea() const;
@@ -369,6 +378,12 @@ public:
         return client_keys;
     }
 
+#ifdef KWIN_BUILD_SCRIPTING
+    Scripting *scripting() {
+        return m_scripting;
+    }
+#endif
+
     /**
      * Returns the list of clients sorted in stacking order, with topmost client
      * at the last position
@@ -389,17 +404,6 @@ public:
     // KDE4 remove me - And it's also in the DCOP interface :(
     void showWindowMenuAt(unsigned long id, int x, int y);
 
-    void toggleCompositing();
-    void loadEffect(const QString& name);
-    void toggleEffect(const QString& name);
-    void reconfigureEffect(const QString& name);
-    void unloadEffect(const QString& name);
-    QString supportInformationForEffect(const QString& name) const;
-    void updateCompositeBlocking(Client* c = NULL);
-
-    QStringList loadedEffects() const;
-    QStringList listOfEffects() const;
-
 
     /**
      * Shows the menu operations menu for the client and makes it active if
@@ -411,7 +415,13 @@ public:
      */
     void showWindowMenu(int x, int y, Client* cl);
     void showWindowMenu(QPoint pos, Client* cl);
-    bool windowMenuShown();
+    const UserActionsMenu *userActionsMenu() const {
+        return m_userActionsMenu;
+    }
+
+#ifdef KWIN_BUILD_KAPPMENU
+    void showApplicationMenu(const QPoint &, const WId);
+#endif
 
     void updateMinimizedOfTransients(Client*);
     void updateOnAllDesktopsOfTransients(Client*);
@@ -434,6 +444,7 @@ public:
     bool hasDecorationShadows() const;
     Qt::Corner decorationCloseButtonCorner();
     bool decorationHasAlpha() const;
+    bool decorationSupportsAnnounceAlpha() const;
     bool decorationSupportsTabbing() const; // Returns true if the decoration supports tabs.
     bool decorationSupportsFrameOverlap() const;
     bool decorationSupportsBlurBehind() const;
@@ -442,31 +453,18 @@ public:
     void cascadeDesktop();
     void unclutterDesktop();
     void doNotManage(const QString&);
+    /**
+     * @deprecated
+     * @todo: remove KDE5
+     **/
     QList<int> decorationSupportedColors() const;
     void nextDesktop();
     void previousDesktop();
     void circulateDesktopApplications();
-    bool compositingActive();
     bool waitForCompositingSetup();
-    void toggleTiling();
-    void nextTileLayout();
-    void previousTileLayout();
     bool stopActivity(const QString &id);
     bool startActivity(const QString &id);
-    QStringList activeEffects() const;
     QString supportInformation() const;
-    bool compositingPossible() const;
-    QString compositingNotPossibleReason() const;
-    bool openGLIsBroken() const;
-    /**
-     * Returns the currently used Compositor (Scene):
-     * @li @c none No Compositing
-     * @li @c xrender XRender
-     * @li @c gl1 OpenGL 1
-     * @li @c gl2 OpenGL 2
-     * @li @c gles OpenGL ES 2
-     **/
-    QString compositingType();
 
     void setCurrentScreen(int new_screen);
 
@@ -508,6 +506,7 @@ public:
     void disableGlobalShortcuts(bool disable);
     void disableGlobalShortcutsForClient(bool disable);
     QPoint cursorPos() const;
+    void checkCursorPos();
 
     void sessionSaveStarted();
     void sessionSaveDone();
@@ -520,38 +519,17 @@ public:
     int packPositionUp(const Client* cl, int oldy, bool top_edge) const;
     int packPositionDown(const Client* cl, int oldy, bool bottom_edge) const;
 
-    static QStringList configModules(bool controlCenter);
-
     void cancelDelayFocus();
     void requestDelayFocus(Client*);
     void updateFocusMousePosition(const QPoint& pos);
     QPoint focusMousePosition() const;
 
     void toggleTopDockShadows(bool on);
-
-    // when adding repaints caused by a window, you probably want to use
-    // either Toplevel::addRepaint() or Toplevel::addWorkspaceRepaint()
-    void addRepaint(const QRect& r);
-    void addRepaint(const QRegion& r);
-    void addRepaint(int x, int y, int w, int h);
-    void checkUnredirect(bool force = false);
-    void checkCompositeTimer();
-    // returns the _estimated_ delay to the next screen update
-    // good for having a rough idea to calculate transformations, bad to rely on.
-    // might happen few ms earlier, might be an entire frame to short. This is NOT deterministic.
-    int nextFrameDelay();
-
-    // Mouse polling
-    void startMousePolling();
-    void stopMousePolling();
-
     Client* getMovingClient() {
         return movingClient;
     }
 
 public slots:
-    void addRepaintFull();
-
     // Keybindings
     void slotSwitchDesktopNext();
     void slotSwitchDesktopPrevious();
@@ -626,8 +604,7 @@ public slots:
 
     void reconfigure();
     void slotReconfigure();
-    void slotReinitCompositing();
-    void resetCompositing();
+    void slotCompositingToggled();
 
     void slotKillWindow();
 
@@ -637,28 +614,12 @@ public slots:
     void slotInvertScreen();
 
     void updateClientArea();
-    void suspendCompositing();
-    void suspendCompositing(bool suspend);
-
-    // NOTE: debug method
-    void dumpTiles() const;
 
     void slotActivateNextTab(); // Slot to move left the active Client.
     void slotActivatePrevTab(); // Slot to move right the active Client.
     void slotUntab(); // Slot to remove the active client from its group.
 
 private slots:
-    void rebuildTabGroupPopup();
-    void rebuildTabListPopup();
-    void entabPopupClient(QAction*);
-    void selectPopupClientTab(QAction*);
-    void desktopPopupAboutToShow();
-    void activityPopupAboutToShow();
-    void clientPopupAboutToShow();
-    void slotSendToDesktop(QAction*);
-    void slotToggleOnActivity(QAction*);
-    void clientPopupActivated(QAction*);
-    void configureWM();
     void desktopResized();
     void screenChangeTimeout();
     void slotUpdateToolWindows();
@@ -668,29 +629,20 @@ private slots:
     void writeWindowRules();
     void slotBlockShortcuts(int data);
     void slotReloadConfig();
-    void setupCompositing();
-    /**
-     * Called from setupCompositing() when the CompositingPrefs are ready.
-     **/
-    void slotCompositingOptionsInitialized();
-    void finishCompositing();
-    void fallbackToXRenderCompositing();
-    void performCompositing();
-    void performMousePoll();
-    void lostCMSelection();
+#ifdef KWIN_BUILD_KAPPMENU
+    void slotShowRequest(qulonglong wid);
+    void slotMenuAvailable(qulonglong wid);
+    void slotMenuHidden(qulonglong wid);
+    void slotClearMenus();
+#endif
     void resetCursorPosTime();
-    void delayedCheckUnredirect();
     void updateCurrentActivity(const QString &new_activity);
     void slotActivityRemoved(const QString &activity);
     void slotActivityAdded(const QString &activity);
     void reallyStopActivity(const QString &id);   //dbus deadlocks suck
     void handleActivityReply();
-    void showHideActivityMenu();
-protected:
-    void timerEvent(QTimerEvent *te);
 
 Q_SIGNALS:
-    Q_SCRIPTABLE void compositingToggled(bool active);
     /**
      * Emitted after the Workspace has setup the complete initialization process.
      * This can be used to connect to for performing post-workspace initialization.
@@ -714,6 +666,7 @@ signals:
                       Qt::KeyboardModifiers modifiers, Qt::KeyboardModifiers oldmodifiers);
     void propertyNotify(long a);
     void configChanged();
+    void reinitializeCompositing();
     /**
      * This signal is emitted when the global
      * activity is changed
@@ -735,16 +688,7 @@ signals:
 private:
     void init();
     void initShortcuts();
-    void initDesktopPopup();
-    void initActivityPopup();
-    void initTabbingPopups();
-    void restartKWin(const QString &reason);
-    void discardPopup();
     void setupWindowShortcut(Client* c);
-    void checkCursorPos();
-#ifdef KWIN_BUILD_ACTIVITIES
-    void updateActivityList(bool running, bool updateCurrent, QString slot = QString());
-#endif
     enum Direction {
         DirectionNorth,
         DirectionEast,
@@ -761,7 +705,7 @@ private:
     bool keepTransientAbove(const Client* mainwindow, const Client* transient);
     void blockStackingUpdates(bool block);
     void updateToolWindows(bool also_hide);
-    void fixPositionAfterCrash(Window w, const XWindowAttributes& attr);
+    void fixPositionAfterCrash(xcb_window_t w, const xcb_get_geometry_reply_t *geom);
     void saveOldScreenSizes();
 
     /// This is the right way to create a new client
@@ -782,16 +726,9 @@ private:
 
     //---------------------------------------------------------------------
 
-    void helperDialog(const QString& message, const Client* c);
-
-    QMenu* clientPopup();
     void closeActivePopup();
     void updateClientArea(bool force);
 
-    bool windowRepaintsPending() const;
-    void setCompositeTimer();
-    int m_timeSinceLastVBlank, m_nextFrameDelay;
-
     typedef QHash< QString, QVector<int> > DesktopFocusChains;
     DesktopFocusChains::Iterator m_desktopFocusChain;
     DesktopFocusChains m_activitiesDesktopFocusChain;
@@ -816,6 +753,11 @@ private:
     static NET::WindowType txtToWindowType(const char* txt);
     static bool sessionInfoWindowTypeMatch(Client* c, SessionInfo* info);
 
+    /**
+     * @returns Whether we have a Compositor and it is active (Scene created)
+     **/
+    bool compositing() const;
+
     Client* active_client;
     Client* last_active_client;
     Client* most_recently_raised; // Used ONLY by raiseOrLowerClient()
@@ -860,31 +802,17 @@ private:
     TabBox::TabBox* tab_box;
 #endif
 
-    QMenu* popup;
-    QMenu* advanced_popup;
-    QMenu* desk_popup;
-    QMenu* activity_popup;
-    QMenu* add_tabs_popup; // Menu to add the group to other group
-    QMenu* switch_to_tab_popup; // Menu to change tab
+    /**
+     * Holds the menu containing the user actions which is shown
+     * on e.g. right click the window decoration.
+     **/
+    UserActionsMenu *m_userActionsMenu;
 
     void modalActionsSwitch(bool enabled);
 
     KActionCollection* keys;
     KActionCollection* client_keys;
     KActionCollection* disable_shortcuts_keys;
-    QAction* mResizeOpAction;
-    QAction* mMoveOpAction;
-    QAction* mMaximizeOpAction;
-    QAction* mShadeOpAction;
-    QAction* mTilingStateOpAction;
-    QAction* mKeepAboveOpAction;
-    QAction* mKeepBelowOpAction;
-    QAction* mFullScreenOpAction;
-    QAction* mNoBorderOpAction;
-    QAction* mMinimizeOpAction;
-    QAction* mCloseOpAction;
-    QAction* mRemoveFromTabGroup; // Remove client from group
-    QAction* mCloseTabGroup; // Close all clients in the group
     ShortcutDialog* client_keys_dialog;
     Client* client_keys_client;
     bool global_shortcuts_disabled;
@@ -931,19 +859,13 @@ private:
     bool forced_global_mouse_grab;
     friend class StackingUpdatesBlocker;
 
-    KSelectionOwner* cm_selection;
-    bool compositingSuspended, compositingBlocked;
-    QBasicTimer compositeTimer;
-    QTimer mousePollingTimer;
-    uint vBlankInterval, fpsInterval;
-    int xrrRefreshRate; // used only for compositing
-    QRegion repaints_region;
     QSlider* transSlider;
     QPushButton* transButton;
-    QTimer unredirectTimer;
-    bool forceUnredirectCheck;
-    QTimer compositeResetTimer; // for compressing composite resets
-    bool m_finishingCompositing; // finishCompositing() sets this variable while shutting down
+
+#ifdef KWIN_BUILD_KAPPMENU
+    //used for menu available before window is mapped
+    QList<WId> m_windowsMenu;
+#endif
 
     Scripting *m_scripting;
 
@@ -1085,11 +1007,6 @@ inline void Workspace::showWindowMenu(int x, int y, Client* cl)
     showWindowMenu(QRect(QPoint(x, y), QPoint(x, y)), cl);
 }
 
-inline bool Workspace::windowMenuShown()
-{
-    return popup && ((QWidget*)popup)->isVisible();
-}
-
 inline void Workspace::setWasUserInteraction()
 {
     was_user_interaction = true;
@@ -1200,17 +1117,6 @@ inline bool Workspace::hasClient(const Client* c)
     return findClient(ClientMatchPredicate(c));
 }
 
-inline void Workspace::checkCompositeTimer()
-{
-    if (!compositeTimer.isActive())
-        setCompositeTimer();
-}
-
-inline int Workspace::nextFrameDelay()
-{
-    return m_nextFrameDelay;
-}
-
 inline bool Workspace::hasDecorationPlugin() const
 {
     if (!mgr) {
@@ -1243,6 +1149,14 @@ inline bool Workspace::decorationHasAlpha() const
     return mgr->factory()->supports(AbilityUsesAlphaChannel);
 }
 
+inline bool Workspace::decorationSupportsAnnounceAlpha() const
+{
+    if (!hasDecorationPlugin()) {
+        return false;
+    }
+    return mgr->factory()->supports(AbilityAnnounceAlphaChannel);
+}
+
 inline bool Workspace::decorationSupportsTabbing() const
 {
     if (!hasDecorationPlugin()) {
