@@ -31,6 +31,7 @@
 #include <KDebug>
 #include <KLocale>
 #include <KStandardDirs>
+#include <KIdleTime>
 
 #include <QtDBus/QDBusConnectionInterface>
 #include <QtDBus/QDBusInterface>
@@ -152,8 +153,8 @@ bool PowermanagementEngine::sourceRequestEvent(const QString &name)
         bool isPlugged = false;
 
         const QList<Solid::Device> list_ac = Solid::Device::listFromType(Solid::DeviceInterface::AcAdapter);
-        foreach (const Solid::Device & device_ac, list_ac) {
-            const Solid::AcAdapter* acadapter = device_ac.as<Solid::AcAdapter>();
+        foreach (Solid::Device device_ac, list_ac) {
+            Solid::AcAdapter* acadapter = device_ac.as<Solid::AcAdapter>();
             isPlugged |= acadapter->isPlugged();
             connect(acadapter, SIGNAL(plugStateChanged(bool,QString)), this,
                     SLOT(updateAcPlugState(bool)), Qt::UniqueConnection);
@@ -187,6 +188,9 @@ bool PowermanagementEngine::sourceRequestEvent(const QString &name)
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
         QObject::connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
                             this, SLOT(screenBrightnessReply(QDBusPendingCallWatcher*)));
+    //any info concerning lock screen/screensaver goes here
+    } else if (name == "UserActivity") {
+        setData("UserActivity", "IdleTime", KIdleTime::instance()->idleTime());
     } else {
         kDebug() << "Data for '" << name << "' not found";
         return false;
@@ -194,6 +198,15 @@ bool PowermanagementEngine::sourceRequestEvent(const QString &name)
     return true;
 }
 
+bool PowermanagementEngine::updateSourceEvent(const QString &source)
+{
+    if (source == "UserActivity") {
+        setData("UserActivity", "IdleTime", KIdleTime::instance()->idleTime());
+        return true;
+    }
+    return Plasma::DataEngine::updateSourceEvent(source);
+}
+
 Plasma::Service* PowermanagementEngine::serviceForSource(const QString &source)
 {
     if (source == "PowerDevil") {
@@ -250,7 +263,6 @@ void PowermanagementEngine::deviceRemoved(const QString& udi)
         QStringList sourceNames(m_batterySources.values());
         sourceNames.removeAll(source);
         setData("Battery", "Sources", sourceNames);
-        setData("Battery", "Has Battery", !sourceNames.isEmpty());
     }
 }
 
@@ -285,7 +297,6 @@ void PowermanagementEngine::deviceAdded(const QString& udi)
             updateBatteryPlugState(battery->isPlugged(), device.udi());
 
             setData("Battery", "Sources", sourceNames);
-            setData("Battery", "Has Battery", !sourceNames.isEmpty());
         }
     }
 }
