@@ -17,7 +17,6 @@
 */
 
 #include "nepomuksearchrunner.h"
-#include "queryclientwrapper.h"
 
 #include <QMenu>
 #include <QMimeData>
@@ -27,13 +26,14 @@
 #include <KDebug>
 #include <KUrl>
 
-#include <nepomuk/nie.h>
-#include <nepomuk/nfo.h>
-#include <Nepomuk/File>
-#include <Nepomuk/Resource>
-#include <Nepomuk/Variant>
-#include <Nepomuk/ResourceManager>
-#include <Nepomuk/Query/QueryServiceClient>
+#include <nepomuk2/nie.h>
+#include <nepomuk2/nfo.h>
+#include <Nepomuk2/File>
+#include <Nepomuk2/Resource>
+#include <Nepomuk2/Variant>
+#include <Nepomuk2/ResourceManager>
+#include <Nepomuk2/Query/ResultIterator>
+#include <Nepomuk2/Query/QueryParser>
 
 #include <KFileItemActions>
 #include <KFileItemList>
@@ -44,7 +44,7 @@
 #include <KMimeTypeTrader>
 #include <KService>
 
-using namespace Nepomuk::Vocabulary;
+using namespace Nepomuk2::Vocabulary;
 
 namespace {
     /**
@@ -54,24 +54,25 @@ namespace {
      * roughly equal the time between key presses of an average user.
      */
     const int s_userActionTimeout = 400;
+    const int s_maxResults = 10;
 }
 
 
-Nepomuk::SearchRunner::SearchRunner( QObject* parent, const QVariantList& args )
+Nepomuk2::SearchRunner::SearchRunner( QObject* parent, const QVariantList& args )
     : Plasma::AbstractRunner( parent, args )
 {
 }
 
 
-Nepomuk::SearchRunner::SearchRunner( QObject* parent, const QString& serviceId )
+Nepomuk2::SearchRunner::SearchRunner( QObject* parent, const QString& serviceId )
     : Plasma::AbstractRunner( parent, serviceId )
 {
 }
 
 
-void Nepomuk::SearchRunner::init()
+void Nepomuk2::SearchRunner::init()
 {
-    Nepomuk::ResourceManager::instance()->init();
+    Nepomuk2::ResourceManager::instance()->init();
 
     // we are pretty slow at times and use DBus calls
     setSpeed(SlowSpeed);
@@ -84,17 +85,17 @@ void Nepomuk::SearchRunner::init()
 }
 
 
-Nepomuk::SearchRunner::~SearchRunner()
+Nepomuk2::SearchRunner::~SearchRunner()
 {
     qDeleteAll(m_konqActions);
 }
 
 
-void Nepomuk::SearchRunner::match( Plasma::RunnerContext& context )
+void Nepomuk2::SearchRunner::match( Plasma::RunnerContext& context )
 {
     kDebug() << &context << context.query();
 
-    if (Nepomuk::ResourceManager::instance()->initialized()) {
+    if( ResourceManager::instance()->initialized() ) {
         // This method needs to be thread-safe since KRunner does simply start new threads whenever
         // the query term changes.
         m_mutex.lock();
@@ -113,16 +114,22 @@ void Nepomuk::SearchRunner::match( Plasma::RunnerContext& context )
         }
 
         // no queries on very short strings
-        if (Query::QueryServiceClient::serviceAvailable() && context.query().count() >= 3) {
-            QueryClientWrapper queryWrapper(this, &context);
-            queryWrapper.runQuery();
-            m_waiter.wakeAll();
+        // We use a default length of 4 characters cause virtuoso has a min length requirement of 4
+        // leading characters. This way we avoid using the expensive regex search
+        if( context.query().length() >= 4 ) {
+            Query::Query query = Query::QueryParser::parseQuery(context.query());
+            query.setLimit(s_maxResults);
+
+            Query::ResultIterator it( query );
+            while( it.next() ) {
+                context.addMatch(context.query(), convertToQueryMatch(it.result()));
+            }
         }
     }
 }
 
 
-void Nepomuk::SearchRunner::run( const Plasma::RunnerContext&, const Plasma::QueryMatch& match )
+void Nepomuk2::SearchRunner::run( const Plasma::RunnerContext&, const Plasma::QueryMatch& match )
 {
     // If no action was selected, the interface doesn't support multiple
     // actions so we simply open the file
@@ -133,16 +140,16 @@ void Nepomuk::SearchRunner::run( const Plasma::RunnerContext&, const Plasma::Que
         }
     }
 
-    Nepomuk::Resource res = match.data().value<Nepomuk::Resource>();
-    KUrl url = res.resourceUri();
+    Nepomuk2::Resource res = match.data().value<Nepomuk2::Resource>();
+    KUrl url = res.uri();
     KUrl nieUrl = res.property( NIE::url() ).toUrl();
     if( !nieUrl.isEmpty() )
         url = nieUrl;
 
     KService::Ptr preferredServicePtr;
-    if (res.hasProperty(Nepomuk::Vocabulary::NIE::mimeType()) &&
-         KUrl(res.property(Nepomuk::Vocabulary::NIE::url()).toUrl()).isLocalFile()) {
-        preferredServicePtr = KMimeTypeTrader::self()->preferredService(res.property(Nepomuk::Vocabulary::NIE::mimeType()).toString());
+    if (res.hasProperty(Nepomuk2::Vocabulary::NIE::mimeType()) &&
+         KUrl(res.property(Nepomuk2::Vocabulary::NIE::url()).toUrl()).isLocalFile()) {
+        preferredServicePtr = KMimeTypeTrader::self()->preferredService(res.property(Nepomuk2::Vocabulary::NIE::mimeType()).toString());
     }
 
     if (preferredServicePtr.isNull() || !KRun::run(*preferredServicePtr.constData(), KUrl::List(url), 0)) {
@@ -150,7 +157,7 @@ void Nepomuk::SearchRunner::run( const Plasma::RunnerContext&, const Plasma::Que
     }
 }
 
-QList<QAction*> Nepomuk::SearchRunner::actionsFromMenu(QMenu *menu, const QString &prefix, QObject *parent)
+QList<QAction*> Nepomuk2::SearchRunner::actionsFromMenu(QMenu *menu, const QString &prefix, QObject *parent)
 {
     Q_ASSERT(menu);
 
@@ -184,7 +191,7 @@ QList<QAction*> Nepomuk::SearchRunner::actionsFromMenu(QMenu *menu, const QStrin
 }
 
 
-QList<QAction*> Nepomuk::SearchRunner::actionsForMatch(const Plasma::QueryMatch &match)
+QList<QAction*> Nepomuk2::SearchRunner::actionsForMatch(const Plasma::QueryMatch &match)
 {
     //Unlike other runners, the actions generated here are likely to see
     //little reuse. Hence, we will clear the actions then generate new
@@ -198,9 +205,9 @@ QList<QAction*> Nepomuk::SearchRunner::actionsForMatch(const Plasma::QueryMatch
     }
     ret << action("open");
 
-    Nepomuk::Resource res = match.data().value<Nepomuk::Resource>();
+    Nepomuk2::Resource res = match.data().value<Nepomuk2::Resource>();
 
-    KUrl url(res.resourceUri());
+    KUrl url(res.uri());
     KIO::UDSEntry entry;
     if (!KIO::NetAccess::stat(url.path(), entry, 0)) {
         return QList<QAction*>();
@@ -225,11 +232,11 @@ QList<QAction*> Nepomuk::SearchRunner::actionsForMatch(const Plasma::QueryMatch
     return ret;
 }
 
-QMimeData * Nepomuk::SearchRunner::mimeDataForMatch(const Plasma::QueryMatch *match)
+QMimeData * Nepomuk2::SearchRunner::mimeDataForMatch(const Plasma::QueryMatch *match)
 {
-    Nepomuk::Resource res = match->data().value<Nepomuk::Resource>();
+    Nepomuk2::Resource res = match->data().value<Nepomuk2::Resource>();
 
-    QUrl url = KUrl(res.property(QUrl("http://www.semanticdesktop.org/ontologies/2007/01/19/nie#url")).toString());
+    QUrl url = res.property(NIE::url()).toUrl();
 
     if (!url.isValid()) {
         return 0;
@@ -243,6 +250,60 @@ QMimeData * Nepomuk::SearchRunner::mimeDataForMatch(const Plasma::QueryMatch *ma
     return result;
 }
 
-K_EXPORT_PLASMA_RUNNER(nepomuksearchrunner, Nepomuk::SearchRunner)
+namespace {
+    qreal normalizeScore(double score) {
+        // no search result is ever a perfect match, NEVER. And mostly, when typing a URL
+        // the users wants to open that url instead of using the search result. Thus, all
+        // search results need to have a lower score than URLs which can drop to 0.5
+        // And in the end, for 10 results, the score is not that important at the moment.
+        // This can be improved in the future.
+        // We go the easy way here and simply cut the score at 0.4
+        return qMin(0.4, score);
+    }
+}
+
+// FIXME: Avoid using the Resource class. Request the extra properties from the query
+Plasma::QueryMatch Nepomuk2::SearchRunner::convertToQueryMatch(const Nepomuk2::Query::Result& result)
+{
+    Plasma::QueryMatch match( this );
+    match.setType(Plasma::QueryMatch::PossibleMatch);
+    match.setRelevance(normalizeScore(result.score()));
+
+    Nepomuk2::Resource res = result.resource();
+
+    QString type;
+    QString iconName;
+
+    KMimeType::Ptr mimetype;
+    if (res.hasProperty(NIE::mimeType())) {
+        mimetype = KMimeType::mimeType(res.property(NIE::mimeType()).toString());
+    }
+    if (!mimetype && res.isFile() && res.toFile().url().isLocalFile()) {
+        const KUrl url(res.toFile().url());
+        mimetype = KMimeType::findByUrl(url);
+    }
+
+    if (mimetype) {
+        type = mimetype->comment();
+        iconName = mimetype->iconName();
+    }
+
+    if (type.isEmpty() ) {
+        type = Nepomuk2::Types::Class(res.type()).label();
+        iconName = res.genericIcon();
+    }
+
+    match.setText(res.genericLabel());
+    match.setSubtext(type);
+    match.setIcon(KIcon(iconName.isEmpty() ? QString::fromLatin1("nepomuk") : iconName));
+
+    match.setData(qVariantFromValue(res));
+    match.setId(KUrl(res.uri()).url());
+
+    return match;
+}
+
+
+K_EXPORT_PLASMA_RUNNER(nepomuksearchrunner, Nepomuk2::SearchRunner)
 
 #include "nepomuksearchrunner.moc"
